/*
 * SDL_Animation son unas librerías para crear animacion en SDL
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Estas librerías tienen licencia Creative Commons y se permite
 * su modificación y utilización libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas información visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cádiz (Spain)
 * javi.pacheco@terra.es
 *
 */
 
 
 #include "SDL_Animation.h"

/*****************************
**
** CLASE FrameObjeto
**
******************************/

SDL_RWops* FrameObjeto::RWFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;
  SDL_RWops *rwops;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	rwops = SDL_RWFromMem(data_ptr, data_size);
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir!: %s - %s\n", fileRar, file); 
  }
  return rwops;
}

void FrameObjeto::load (char *path) {
	
	SDL_Surface *tmpImg;
	
	string s = path;
	
	string ext = s.substr(s.length()-4, 4);
	
	if (strcmp(ext.data(), "timp") == 0) {
		char fileRar[127];
		sprintf (fileRar, "%s/%s", DATA_DIR, archivoComprimido);
		SDL_RWops *rw;
		rw = RWFromRar(fileRar, path);
		tmpImg = IMG_Load_RW(rw, 0);
		img = SDL_DisplayFormat(tmpImg);
	} else {
		tmpImg = IMG_Load (path);
		img = SDL_DisplayFormat(tmpImg);
		if ( img == NULL) {
			printf ("No se cargo la imagen: %s\n", path);
		}
	}
	
	SDL_FreeSurface (tmpImg);
	
}

void FrameObjeto::loadAlpha (char *path) {
	// comprobar si es un archivo .timp
	// si es así lo descomprimimos del rar
	// sino lo buscamos directamente
	
	SDL_Surface *tmpImg;
	
	string s = path;
	
	string ext = s.substr(s.length()-4, 4);
	
	if (strcmp(ext.data(), "timp") == 0) {
		char fileRar[127];
		sprintf (fileRar, "%s/%s", DATA_DIR, archivoComprimido);
		SDL_RWops *rw;
		rw = RWFromRar(fileRar, path);
		tmpImg = IMG_Load_RW(rw, 0);
		img = SDL_DisplayFormatAlpha(tmpImg);
		if ( img == NULL) {
			printf ("No se cargo la imagen: %s\n", path);
		}
	} else {
		tmpImg = IMG_Load (path);
		img = SDL_DisplayFormatAlpha(tmpImg);
		if ( img == NULL) {
			printf ("No se cargo la imagen: %s\n", path);
		}
	}
	
	SDL_FreeSurface (tmpImg);	
	
	
}

void FrameObjeto::load (char *path, SDL_Surface *sc) {
	
	SDL_Surface *tmpImg;

	tmpImg = IMG_Load (path);
	if ( tmpImg == NULL) {
		printf ("No se cargo la imagen: %s\n", path);
	}
	img = SDL_ConvertSurface ( tmpImg, sc->format, SDL_SWSURFACE);
	SDL_FreeSurface(tmpImg);

	}

void FrameObjeto::loadSurface (SDL_Surface *surface) {
	img = SDL_DisplayFormat(surface);
}

void FrameObjeto::unLoad() {
	SDL_FreeSurface (img);	
}

/*****************************
**
** CLASE OBJETO
**
******************************/

Objeto::Objeto() {
	fantasma = 0;
	int nf=1;
	actualFrame = 0;
	tipoCoordenadas = COORDENADASPOLARES;
	rebote = 1;
	numMovimientos = 0;
	ciclo = 0;
	movimientoActual = -1;
	cicloNacimiento = -1;
	cicloMuerte = -1;
	ciclosEntreFrames = 0;
	tiempoFadeIn = 0;
	fadein_factor_begin = 0;
	fadein_factor_end = 0;
	tiempoFadeOut = 0;
	fadeout_factor_begin = 0;
	fadeout_factor_end = 0;
	tiempoFadeInTotal = 0;
	tiempoFadeOutTotal = 0;
	cortinillaHIn = 0;
	cortinillaVIn = 0;
	cortinillaHOut = 0;
	cortinillaVOut = 0;
	curtainVInUpDown = 1;
	curtainHInLeftRight = 1;
	curtainVOutUpDown = 1;
	curtainHOutLeftRight = 1;
	zoomin = -1;
	zoomout = -1;
	factor_zoom_actual = 1;
	rotatein = -1;
	rotateout = -1;
	angle_rotate_actual = -1;
	align = 0;
	valign = 0;
	bgR = -1;
	bgG = -1;
	bgB = -1;
	imagen_conAlpha = 0;
	isAnimacionParada = 0;
	
	sprite = new FrameObjeto[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	cont = 0;
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = -1;
	strcpy(nombre, "");
}

Objeto::Objeto (int nf) {
	fantasma = 0;
	actualFrame = 0;
	tipoCoordenadas = COORDENADASPOLARES;
	rebote = 1;
	numMovimientos = 0;
	ciclo = 0;
	movimientoActual = -1;
	cicloNacimiento = -1;
	cicloMuerte = -1;
	ciclosEntreFrames = 0;
	tiempoFadeIn = 0;
	tiempoFadeOut = 0;
	tiempoFadeInTotal = 0;
	tiempoFadeOutTotal = 0;
	cortinillaHIn = 0;
	cortinillaVIn = 0;
	cortinillaHOut = 0;
	cortinillaVOut = 0;
	curtainVInUpDown = 1;
	curtainHInLeftRight = 1;
	curtainVOutUpDown = 1;
	curtainHOutLeftRight = 1;
	zoomin = -1;
	zoomout = -1;
	factor_zoom_actual = 1;
	rotatein = -1;
	rotateout = -1;
	angle_rotate_actual = -1;
	align = 0;
	valign = 0;
	bgR = -1;
	bgG = -1;
	bgB = -1;
	imagen_conAlpha = 0;
	isAnimacionParada = 0;
	
	sprite = new FrameObjeto[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	cont = 0;	
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = -1;
	strcpy(nombre, "");
}

Objeto::Objeto (int nf, int ni) {
	fantasma = 0;
	actualFrame = 0;
	tipoCoordenadas = COORDENADASPOLARES;
	numMovimientos = 0;
	ciclo = 0;
	movimientoActual = -1;
	cicloNacimiento = -1;
	cicloMuerte = -1;
	ciclosEntreFrames = 0;
	tiempoFadeIn = 0;
	tiempoFadeOut = 0;
	tiempoFadeInTotal = 0;
	tiempoFadeOutTotal = 0;
	cortinillaHIn = 0;
	cortinillaVIn = 0;
	cortinillaHOut = 0;
	cortinillaVOut = 0;
	curtainVInUpDown = 1;
	curtainHInLeftRight = 1;
	curtainVOutUpDown = 1;
	curtainHOutLeftRight = 1;
	zoomin = -1;
	zoomout = -1;
	factor_zoom_actual = 1;
	rotatein = -1;
	rotateout = -1;
	angle_rotate_actual = -1;
	align = 0;
	valign = 0;
	bgR = -1;
	bgG = -1;
	bgB = -1;
	imagen_conAlpha = 0;
	isAnimacionParada = 0;
	
	sprite = new FrameObjeto[nf];
	transparente = new Uint32[nf];
	nFrames = nf;
	cont = 0;	
	activo = 0;
	oldX = 0;
	oldY = 0;
	alpha = -1;
	strcpy(nombre, "");
}

bool Objeto::fueraLimitesDePantalla() {
	if ( ( getY() < getLimiteSup() ) || ( getY() > getLimiteInf() ) || ( getX() < getLimiteIzq() ) || ( getX() > getLimiteDer() ) ){ 
		return 1;
	} else {
		return 0;
	}
}

void Objeto::setMovimiento (int cicloini, int ciclofin, int x, int y, int xFin, int yFin, int desaparecerAlFinal, int tiempoFadeIn, int fadein_opacidad_begin, int fadein_opacidad_end, int tiempoFadeOut, int fadeout_opacidad_begin, int fadeout_opacidad_end, int tiempoCortinillaHIn, int tiempoCortinillaHOut, int tiempoCortinillaVIn, int tiempoCortinillaVOut) {
	// si es el primer movimiento actualizamos la variable cicloNacimiento
	if (numMovimientos == 0) {
		setCicloNacimiento(cicloini);
	}
	if (desaparecerAlFinal == 1) {
		setCicloMuerte (ciclofin);
	}
	
	Movimiento mov;
	
	mov.cicloini = cicloini;
	mov.ciclofin = ciclofin;
	mov.posxini = x;
	mov.posyini = y;
	mov.posxfin = xFin;
	mov.posyfin = yFin;
	mov.tiempo_fadein = tiempoFadeIn;
	mov.fadein_factor_begin = fadein_opacidad_begin;
	mov.fadein_factor_end = fadein_opacidad_end;
	mov.fadeout_factor_begin = fadeout_opacidad_begin;
	mov.fadeout_factor_end = fadeout_opacidad_end;
	mov.tiempo_fadeout = tiempoFadeOut;
	mov.tiempo_cortinillaH_in = tiempoCortinillaHIn;
	mov.tiempo_cortinillaH_out = tiempoCortinillaHOut;
	mov.tiempo_cortinillaV_in = tiempoCortinillaVIn;
	mov.tiempo_cortinillaV_out = tiempoCortinillaVOut;
	mov.tiempo_zoom_in = -1;
	mov.tiempo_zoom_out = -1;
	mov.tiempo_rotate_in = -1;
	mov.tiempo_rotate_out = -1;
	
	movimientos.push_back(mov);

	numMovimientos++;
}

// Las 4 siguientes funciones solo pueden ser llamadas
// una vez llamada la funcion anterior

void Objeto::setCortinillaVIn_EnMovimientoActual(int tiempoCortinillaVIn, bool UpDown) {
	movimientos[numMovimientos-1].tiempo_cortinillaV_in = tiempoCortinillaVIn;
	movimientos[numMovimientos-1].curtainVInUpDown = UpDown;
}

void Objeto::setCortinillaHIn_EnMovimientoActual(int tiempoCortinillaHIn, bool LeftRight) {
	movimientos[numMovimientos-1].tiempo_cortinillaH_in = tiempoCortinillaHIn;
	movimientos[numMovimientos-1].curtainHInLeftRight = LeftRight;
}

void Objeto::setCortinillaVOut_EnMovimientoActual(int tiempoCortinillaVOut, bool UpDown) {
	movimientos[numMovimientos-1].tiempo_cortinillaV_out = tiempoCortinillaVOut;
	movimientos[numMovimientos-1].curtainVOutUpDown = UpDown;
}

void Objeto::setCortinillaHOut_EnMovimientoActual(int tiempoCortinillaHOut, bool LeftRight) {
	movimientos[numMovimientos-1].tiempo_cortinillaH_out = tiempoCortinillaHOut;
	movimientos[numMovimientos-1].curtainHOutLeftRight = LeftRight;
}

void Objeto::setZoomIn_EnMovimientoActual(int tiempoZoomIn, double factor_begin, double factor_end) {
	movimientos[numMovimientos-1].tiempo_zoom_in = tiempoZoomIn;
	movimientos[numMovimientos-1].tiempo_zoom_in_factor_begin = factor_begin;
	movimientos[numMovimientos-1].tiempo_zoom_in_factor_end = factor_end;
}

void Objeto::setZoomOut_EnMovimientoActual(int tiempoZoomOut, double factor_begin, double factor_end) {
	movimientos[numMovimientos-1].tiempo_zoom_out = tiempoZoomOut;
	movimientos[numMovimientos-1].tiempo_zoom_out_factor_begin = factor_begin;
	movimientos[numMovimientos-1].tiempo_zoom_out_factor_end = factor_end;
}

void Objeto::setRotateIn_EnMovimientoActual(int tiempoRotateIn, int angle_begin, int angle_end, int factor) {
	movimientos[numMovimientos-1].tiempo_rotate_in = tiempoRotateIn;
	movimientos[numMovimientos-1].rotatein_angle_begin = angle_begin;
	movimientos[numMovimientos-1].rotatein_angle_end = angle_end;
	movimientos[numMovimientos-1].factorRotatein = factor;
}

void Objeto::setRotateOut_EnMovimientoActual(int tiempoRotateOut, int angle_begin, int angle_end, int factor) {
	movimientos[numMovimientos-1].tiempo_rotate_out = tiempoRotateOut;
	movimientos[numMovimientos-1].rotateout_angle_begin = angle_begin;
	movimientos[numMovimientos-1].rotateout_angle_end = angle_end;
	movimientos[numMovimientos-1].factorRotateout = factor;
}

void Objeto::unLoad() {
	int i;
	for (i=0; i<nFrames-1; i++)
		sprite[i].unLoad();
}

void Objeto::addFrame (FrameObjeto frame) {
	if (cont<nFrames) {
		transparente[cont] = SDL_GetPixel (frame.img, 0,0); // guarda el valor transparente del frame
		sprite[cont]=frame;
		cont++;
	}
}

void Objeto::modificarFrameN (FrameObjeto frame, int n) {
	if (n<nFrames) {
		sprite[n]=frame;
	}
}

void Objeto::selFrame (int nf) {
	if (nf<=nFrames) {
		actualFrame = nf;
	}
}

void Objeto::selSiguienteFrame () {
	if (actualFrame<cont-1) {
		actualFrame++;
	} else {
		actualFrame = 0;
	}
}

void Objeto::selSiguienteFrame_Rebote () {
	if (actualFrame>cont-2) {
		rebote=-1;
	}
	if (actualFrame<1) {
		rebote=1;
	}
	actualFrame += rebote;
}

void Objeto::selSiguienteFrame_ReboteHasta (int hasta) {
	if (actualFrame>hasta-2) {
		rebote=-1;
	}
	if (actualFrame<1) {
		rebote=1;
	}
	actualFrame += rebote;
}

void Objeto::conmutaFantasma () {
	if ( fantasma == 0 ) {
		fantasma = 1;
	} else {
		fantasma = 0;
	}	
}

void Objeto::comprobarMovimiento () { 
	int i, ma;
	
	// comprobar paradas en la animacion
	
	for (i=0; i<controlAnimation.size(); i++) {
		if (controlAnimation[i].cycle == ciclo) {
			if (controlAnimation[i].isPlay) {
				playAnimacion();
			} else {
				stopAnimacion();
				if ( (controlAnimation[i].frame>0) && (controlAnimation[i].frame<nFrames) ) {
					selFrame(controlAnimation[i].frame-1);
				}
			}
		}
	}	

	// comprobando movimiento
	
	ma = movimientoActual;
	for (i = 0; i<movimientos.size(); i++) {
		if ((movimientos[i].cicloini<=ciclo) && (movimientos[i].ciclofin>ciclo)) {
			ma = i;
		}
	}
	float distanciaEntreElementos;
	float coseno, seno;
	
	if (ma != movimientoActual) {
		setX(movimientos[ma].posxini);
		setY(movimientos[ma].posyini);
		// calcular coseno para coordenadas polares
		distanciaEntreElementos = sqrt( (double) (movimientos[ma].posyfin-movimientos[ma].posyini)*(movimientos[ma].posyfin-movimientos[ma].posyini) + (movimientos[ma].posxfin-movimientos[ma].posxini)*(movimientos[ma].posxfin-movimientos[ma].posxini));
		coseno = (float)(movimientos[ma].posyfin-movimientos[ma].posyini)/ distanciaEntreElementos;
		seno = (float)(movimientos[ma].posxfin-movimientos[ma].posxini)/ distanciaEntreElementos;
		setDPolarSeno(seno);
		setDPolarCoseno(coseno);
		setDPolarDistancia(distanciaEntreElementos/abs(movimientos[ma].ciclofin-movimientos[ma].cicloini));
		if (movimientos[ma].tiempo_fadein>0) {
			setFadeIn(movimientos[ma].tiempo_fadein, movimientos[ma].fadein_factor_begin, movimientos[ma].fadein_factor_end);
		}
		if (movimientos[ma].tiempo_cortinillaH_in>0) {
			setCortinillaHIn(movimientos[ma].tiempo_cortinillaH_in, movimientos[ma].curtainHInLeftRight);
		}
		if (movimientos[ma].tiempo_cortinillaV_in>0) {
			setCortinillaVIn(movimientos[ma].tiempo_cortinillaV_in, movimientos[ma].curtainVInUpDown);
		}
		if (movimientos[ma].tiempo_zoom_in>0) {
			setZoomIn(movimientos[ma].tiempo_zoom_in);
			setZoomInFactorBegin(movimientos[ma].tiempo_zoom_in_factor_begin);
			setZoomInFactorEnd(movimientos[ma].tiempo_zoom_in_factor_end);
		}
		if (movimientos[ma].tiempo_rotate_in>0) {
			setRotateIn(movimientos[ma].tiempo_rotate_in);
			setRotateInAngles(movimientos[ma].rotatein_angle_begin, movimientos[ma].rotatein_angle_end, movimientos[ma].factorRotatein);
		}
	}

	movimientoActual = ma;

}

void Objeto::mover () { 
	comprobarMovimiento();
	
	if (tipoCoordenadas == COORDENADASPOLARES) {
		if (movimientoActual>=0) {
			// para la derecha
			if ((movimientos[movimientoActual].posxfin>movimientos[movimientoActual].posxini) && (movimientos[movimientoActual].posxfin>posX)) {
				posX += polarDistancia*polarSeno;
			}
			// para la izquierda
			if ((movimientos[movimientoActual].posxfin<movimientos[movimientoActual].posxini) && (movimientos[movimientoActual].posxfin<posX)) {
				posX += polarDistancia*polarSeno;
			}
			// para abajo
			if ((movimientos[movimientoActual].posyfin>movimientos[movimientoActual].posyini) && (movimientos[movimientoActual].posyfin>posY)) {
				posY += polarDistancia*polarCoseno;
			}
			// para arriba
			if ((movimientos[movimientoActual].posyfin<movimientos[movimientoActual].posyini) && (movimientos[movimientoActual].posyfin<posY)) {
				posY += polarDistancia*polarCoseno;
			}
		}
	} else {
		posX += dx; 
		posY += dy; 
	}
	// problema inexplicable
	// si el movimientoActual es -1 se pone a 0 para hacer los OUT
	int movimientoActualAux = movimientoActual; 
	if (movimientoActual == -1) {
		movimientoActual = 0;
	}
	// comprobar si tiene que hacer FadeOut
	if ((movimientos[movimientoActual].tiempo_fadeout > 0) && (movimientos[movimientoActual].ciclofin-movimientos[movimientoActual].tiempo_fadeout==ciclo)) {
		setFadeOut(movimientos[movimientoActual].tiempo_fadeout, movimientos[movimientoActual].fadeout_factor_begin, movimientos[movimientoActual].fadeout_factor_end);
	}
	// comprobar si tiene que hacer cortinilla de salida
	if ((movimientos[movimientoActual].tiempo_cortinillaH_out > 0) && (movimientos[movimientoActual].ciclofin-movimientos[movimientoActual].tiempo_cortinillaH_out==ciclo)) {
		setCortinillaHOut(movimientos[movimientoActual].tiempo_cortinillaH_out, movimientos[movimientoActual].curtainHOutLeftRight);
	}
	if ((movimientos[movimientoActual].tiempo_cortinillaV_out > 0) && (movimientos[movimientoActual].ciclofin-movimientos[movimientoActual].tiempo_cortinillaV_out==ciclo)) {
		setCortinillaVOut(movimientos[movimientoActual].tiempo_cortinillaV_out, movimientos[movimientoActual].curtainVOutUpDown);
	}
	// comprobar si tiene que hacer zoomout
	if ((movimientos[movimientoActual].tiempo_zoom_out > 0) && (movimientos[movimientoActual].ciclofin-movimientos[movimientoActual].tiempo_zoom_out==ciclo)) {
		setZoomOut(movimientos[movimientoActual].tiempo_zoom_out);
		setZoomOutFactorBegin(movimientos[movimientoActual].tiempo_zoom_out_factor_begin);
		setZoomOutFactorEnd(movimientos[movimientoActual].tiempo_zoom_out_factor_end);
	}
	// comprobar si tiene que hacer rotateout
	if ((movimientos[movimientoActual].tiempo_rotate_out > 0) && (movimientos[movimientoActual].ciclofin-movimientos[movimientoActual].tiempo_rotate_out==ciclo)) {
		setRotateOut(movimientos[movimientoActual].tiempo_rotate_out);
		setRotateOutAngles(movimientos[movimientoActual].rotateout_angle_begin, movimientos[movimientoActual].rotateout_angle_end, movimientos[movimientoActual].factorRotateout);
	}
	movimientoActual = movimientoActualAux;
	ciclo++;
}

void Objeto::dibujar (SDL_Surface *superficie) {
	SDL_Surface *surfaceRotoZoom;
	bool blitSurfaceRotoZoom = 0;
	SDL_Rect src, dest;

	src.x = 0;
	src.y = 0;
	src.w = getAncho();
	src.h = getAlto();
	
	int incrementoX_dest = 0;
	int incrementoY_dest = 0;



	// trabajo con cortinillas
	
	if (cortinillaHIn>0) {
		if (curtainHInLeftRight) { // entrada de izquierda a derecha
			//if (getAncho() < maxCortinillaHIn) {
			//	src.w = (maxCortinillaHIn - cortinillaHIn);
			//} else {
				src.w = (getAncho()/maxCortinillaHIn) * (maxCortinillaHIn - cortinillaHIn);
			//}
		} else { // entrada de derecha a izquierda
			//if (getAncho() < maxCortinillaHIn) {
			//	src.w = (maxCortinillaHIn - cortinillaHIn);
			//} else {
				src.x = getAncho() - (getAncho()/maxCortinillaHIn) * (maxCortinillaHIn - cortinillaHIn);
				incrementoX_dest = getAncho() - (getAncho()/maxCortinillaHIn) * (maxCortinillaHIn - cortinillaHIn);
			//}
		}
		cortinillaHIn--;
	}
	
	if (cortinillaVIn>0) {
		if (curtainVInUpDown) { // entrada de Arriba a abajo
			//if (getAlto() < maxCortinillaVIn) {
			//	src.h = (maxCortinillaVIn - cortinillaVIn);
			//} else {
				src.h = (getAlto()/maxCortinillaVIn) * (maxCortinillaVIn - cortinillaVIn);
			//}
		} else {
			//if (getAlto() < maxCortinillaVIn) {
			//	src.h = (maxCortinillaVIn - cortinillaVIn);
			//} else {
				src.y = getAlto() - (getAlto()/maxCortinillaVIn) * (maxCortinillaVIn - cortinillaVIn);
				incrementoY_dest = getAlto() - (getAlto()/maxCortinillaVIn) * (maxCortinillaVIn - cortinillaVIn);
			//}
		}
		cortinillaVIn--;
	}
	
	if (cortinillaHOut>0) {
		if (curtainHOutLeftRight) { // salida de izquierda a derecha
			//if (getAncho() < maxCortinillaHOut) {
			//	src.w = getAncho() - (maxCortinillaHOut - cortinillaHOut);
			//} else {
				src.x = ((getAncho()/maxCortinillaHOut) * (maxCortinillaHOut - cortinillaHOut));
				incrementoX_dest = ((getAncho()/maxCortinillaHOut) * (maxCortinillaHOut - cortinillaHOut));
			//}
		} else {
			//if (getAncho() < maxCortinillaHOut) {
			//	src.w = getAncho() - (maxCortinillaHOut - cortinillaHOut);
			//} else {
				src.w = getAncho() - ((getAncho()/maxCortinillaHOut) * (maxCortinillaHOut - cortinillaHOut));
			//}
		}
		
		cortinillaHOut--;
		if (cortinillaHOut <= 1) {
			alpha = 0;
		}
	}
	
	if (cortinillaVOut>0) {
		if (curtainVOutUpDown) { // salida de arriba a abajo
			//if (getAlto() < maxCortinillaVOut) {
			//	src.h = getAlto() - (maxCortinillaVOut - cortinillaVOut);
			//} else {
				src.y = ((getAlto()/maxCortinillaVOut) * (maxCortinillaVOut - cortinillaVOut));
				incrementoY_dest = ((getAlto()/maxCortinillaVOut) * (maxCortinillaVOut - cortinillaVOut));
			//}
		} else { // salida de abajo a arriba
			//if (getAlto() < maxCortinillaVOut) {
			//	src.h = getAlto() - (maxCortinillaVOut - cortinillaVOut);
			//} else {
				src.h = getAlto() - ((getAlto()/maxCortinillaVOut) * (maxCortinillaVOut - cortinillaVOut));
			//}
		}
		cortinillaVOut--;
		if (cortinillaVOut <= 1) {
			alpha = 0;
		}
	}
	
	// trabajo con zoom

	if ( (factor_zoom_actual != 1) && (zoomin<0) && (zoomout<0) ) { // en el caso que el zoom quedé mayor o menos al tamaño original
		surfaceRotoZoom = zoomSurface (sprite[actualFrame].img, factor_zoom_actual, factor_zoom_actual, 0);
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
	}
	
	if (zoomin>=0) {
		double tam, multiplo;
		multiplo = (zoomin_factor_end -zoomin_factor_begin) / maxZoomin;
		tam = zoomin_factor_begin + (multiplo*((double)maxZoomin-zoomin));
		surfaceRotoZoom = zoomSurface (sprite[actualFrame].img, tam, tam, 0);
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
		zoomin--;
		if ((zoomin == 0) && (zoomin_factor_end!=1)) {
			factor_zoom_actual = tam;
		}
	}
	
	if (zoomout>=0) {
		factor_zoom_actual = 1;
		double tam, multiplo;
		multiplo = (zoomout_factor_end -zoomout_factor_begin) / maxZoomout;
		tam = zoomout_factor_begin + (multiplo*((double)maxZoomout-zoomout));
		surfaceRotoZoom = zoomSurface (sprite[actualFrame].img, tam, tam, 0);
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
		zoomout--;
		if ((zoomout == 0) && (zoomout_factor_end!=1)) {
			factor_zoom_actual = tam;
			zoomout = -1;
		}
	}
	
	// hacer rotaciones

	if ( (angle_rotate_actual != -1) && (rotatein<=0) && (rotateout<=0) ) { // la rotacion no ha quedado a 0 grados
		if (blitSurfaceRotoZoom == 1) { // si ya ha hecho zoom utilizamos surfaceRotoZoom
			surfaceRotoZoom = rotozoomSurface (surfaceRotoZoom, angle_rotate_actual, 1, 0);
		} else {
			surfaceRotoZoom = rotozoomSurface (sprite[actualFrame].img, angle_rotate_actual, 1, 0);
		}		
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
	}
	
	double angulo;
	
	if (rotatein>0) {
		if (factorRotatein>0) {
			angulo = ((double)(rotatein_angle_begin - rotatein_angle_end)/maxRotatein) * (maxRotatein-(rotatein-1));
		} else {
			angulo = rotatein_angle_begin - ((double)(rotatein_angle_begin - rotatein_angle_end)/maxRotatein) * (maxRotatein-(rotatein-1));
		}
		if (blitSurfaceRotoZoom == 1) { // si ya ha hecho zoom utilizamos surfaceRotoZoom
			surfaceRotoZoom = rotozoomSurface (surfaceRotoZoom, (int)angulo, 1, 0);
		} else {
			surfaceRotoZoom = rotozoomSurface (sprite[actualFrame].img, (int)angulo, 1, 0);
		}		
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
		rotatein--;
		if (rotatein == 0) {
			if (angulo == 0) {
				angle_rotate_actual = -1;
			} else if ((int)angulo%360!=0) {
				angle_rotate_actual = (int)angulo;
			} else {
				angle_rotate_actual = -1;
			}
		}
	}		

	if (rotateout>0) {
		//double angulo = ((double)(rotateout_angle_begin - rotateout_angle_end)/maxRotatein) * (maxRotateout-(rotateout-1));
		if (factorRotateout>0) {
			angulo = ((double)(rotateout_angle_begin - rotateout_angle_end)/maxRotatein) * (maxRotateout-(rotateout-1));
		} else {
			angulo = rotateout_angle_begin - ((double)(rotateout_angle_begin - rotateout_angle_end)/maxRotatein) * (maxRotateout-(rotateout-1));	
		}
		if (blitSurfaceRotoZoom == 1) { // si ya ha hecho zoom utilizamos surfaceRotoZoom
			surfaceRotoZoom = rotozoomSurface (surfaceRotoZoom, (int)angulo, 1, 0);
		} else {
			surfaceRotoZoom = rotozoomSurface (sprite[actualFrame].img, (int)angulo, 1, 0);
		}		
		src.h = surfaceRotoZoom->h;
		src.w = surfaceRotoZoom->w;
		blitSurfaceRotoZoom = 1;
		rotateout--;
		if (rotateout == 0) {
			if (angulo == 0) {
				angle_rotate_actual = -1;
			} else if ((int)angulo%360!=0) {
				angle_rotate_actual = (int)angulo;
			} else {
				angle_rotate_actual = -1;
			}
		}
	}		
	
	// en el caso de animaciones. Cambiar imagenes
	
	if ( (nFrames > 1) && (!isAnimacionParada) ) {
		if (ciclo%ciclosEntreFrames==0) {
			if (tipoRotacion==ROTACION_CIRCULAR) {
				selSiguienteFrame();
			} else {
				selSiguienteFrame_Rebote();
			}
		}
	}

	// hacer fades
	int factor_opacidad = 0;
	if (tiempoFadeIn > 0) {
		tiempoFadeIn--;
		if (fadein_factor_begin < fadein_factor_end) {
			factor_opacidad = fadein_factor_end - fadein_factor_begin;
			alpha = (int) fadein_factor_begin + ( (factor_opacidad/tiempoFadeInTotal) * (tiempoFadeInTotal - tiempoFadeIn) );
		} else {
			factor_opacidad = fadein_factor_begin - fadein_factor_end;
			alpha = (int) fadein_factor_begin - ( (factor_opacidad/tiempoFadeInTotal) * (tiempoFadeInTotal - tiempoFadeIn) );
		}
		if (tiempoFadeIn == 0) {
			alpha = fadein_factor_end;
		}
	}
	if (tiempoFadeOut > 0) {
		tiempoFadeOut--;
		if (fadeout_factor_begin < fadeout_factor_end) {
			factor_opacidad = fadeout_factor_end - fadeout_factor_begin;
			alpha = (int) fadeout_factor_begin + ( (factor_opacidad/tiempoFadeOutTotal) * (tiempoFadeOutTotal - tiempoFadeOut) );
		} else {
			factor_opacidad = fadeout_factor_begin - fadeout_factor_end;
			alpha = (int) fadeout_factor_begin - ( (factor_opacidad/tiempoFadeOutTotal) * (tiempoFadeOutTotal - tiempoFadeOut) );
		}
		if (tiempoFadeOut == 0) {
			alpha = fadeout_factor_end;
		}
	}

	// alineacion
	int altura, anchura;
	if (blitSurfaceRotoZoom==1) {
		altura = surfaceRotoZoom->h;
		anchura = surfaceRotoZoom->w;
	} else {
		altura = sprite[actualFrame].img->h;
		anchura = sprite[actualFrame].img->w;
	}
	

	switch (align)
	{
		case  1:
			dest.x = (int)posX - anchura/2 + incrementoX_dest;
			break;
		case  2:
			dest.x = (int)posX - anchura + incrementoX_dest;
			break;
		default:
			dest.x = (int)posX + incrementoX_dest;
			break;
	}
	
	switch (valign)
	{
		case  1:
			dest.y = (int)posY - altura/2 + incrementoY_dest;
			break;
		case  2:
			dest.y = (int)posY - altura + incrementoY_dest;
			break;
		default:
			dest.y = (int)posY + incrementoY_dest;
			break;
	}

	SDL_Surface* sdl_surface_copy;

	if (blitSurfaceRotoZoom==1) {

		if ( (alpha < 255) && (alpha >= 0) ) {
			sdl_surface_copy = SDL_CreateRGBSurface (surfaceRotoZoom->flags,
											surfaceRotoZoom->w, surfaceRotoZoom->h, surfaceRotoZoom->format->BitsPerPixel,
											surfaceRotoZoom->format->Rmask, surfaceRotoZoom->format->Gmask,
											surfaceRotoZoom->format->Bmask,
											0);

			SDL_SetColorKey(sdl_surface_copy, SDL_SRCCOLORKEY, (int)transparente[actualFrame]);
			SDL_FillRect(sdl_surface_copy, NULL, (int)transparente[actualFrame]);
			//int colorkey = SDL_MapRGB(sdl_surface_copy->format, bgR, bgG, bgB);
			//SDL_FillRect(sdl_surface_copy, NULL, colorkey);
			//SDL_SetColorKey(sdl_surface_copy, SDL_SRCCOLORKEY, colorkey);
		
		
			SDL_BlitSurface(surfaceRotoZoom, NULL, sdl_surface_copy, NULL);
			SDL_SetAlpha(sdl_surface_copy ,SDL_SRCALPHA,alpha);
			SDL_BlitSurface (sdl_surface_copy, &src, superficie, &dest);
			SDL_FreeSurface (sdl_surface_copy);
		} else {
			SDL_BlitSurface (surfaceRotoZoom, &src, superficie, &dest);
		}

	} else {
		if (imagen_conAlpha == 1) {
			if ( (alpha < 255) && (alpha >= 0) ) {
				sdl_surface_copy = SDL_CreateRGBSurface (sprite[actualFrame].img->flags,
									sprite[actualFrame].img->w, sprite[actualFrame].img->h, sprite[actualFrame].img->format->BitsPerPixel,
									sprite[actualFrame].img->format->Rmask, sprite[actualFrame].img->format->Gmask,
									sprite[actualFrame].img->format->Bmask,
									0);
				SDL_SetColorKey(sdl_surface_copy, SDL_SRCCOLORKEY, (int)transparente[actualFrame]);
				SDL_FillRect(sdl_surface_copy, NULL, (int)transparente[actualFrame]);
				//int colorkey = SDL_MapRGB(sdl_surface_copy->format, bgR, bgG, bgB);
				//SDL_FillRect(sdl_surface_copy, NULL, colorkey);
				//SDL_SetColorKey(sdl_surface_copy, SDL_SRCCOLORKEY, colorkey);
			
			
				SDL_BlitSurface(sprite[actualFrame].img, NULL, sdl_surface_copy, NULL);
				SDL_SetAlpha(sdl_surface_copy ,SDL_SRCALPHA,alpha);
				SDL_BlitSurface (sdl_surface_copy, &src, superficie, &dest);
				SDL_FreeSurface (sdl_surface_copy);
			} else {
				SDL_BlitSurface (sprite[actualFrame].img, &src, superficie, &dest);
			}
		} else {
			SDL_SetAlpha(sprite[actualFrame].img, SDL_SRCALPHA, alpha);
			SDL_BlitSurface (sprite[actualFrame].img, &src, superficie, &dest);
		}
	}
}

Uint32 Objeto::SDL_GetPixel(SDL_Surface *surface, int x, int y) {
	int bpp = surface->format->BytesPerPixel;
	/* Here p is the address to the pixel we want to retrieve */
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	switch(bpp)
	{
		case(1):
		return *p;

		case(2):
		return *(Uint16 *)p;

		case(3):
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case(4):
			return *(Uint32 *)p;

		default:
			return 0;       /* shouldn't happen, but avoids warnings */
	}
}


/*****************************
**
** CLASE SONIDO
**
******************************/

SDL_RWops* Sonido::RWFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;
  SDL_RWops *rwops;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	rwops = SDL_RWFromMem(data_ptr, data_size);
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir sonido!\n"); 
  }
  return rwops;
}


Sonido::Sonido(char archivo[127], int volumen) {
	
	string s = archivo;
	
	string ext = s.substr(s.length()-4, 4);
	
	if (strcmp(ext.data(), "tsog") == 0) {
		char fileRar[127];
		sprintf (fileRar, "%s/%s", DATA_DIR, archivoComprimido);
		musica = Mix_LoadWAV_RW (RWFromRar(fileRar, archivo), 1);
		if ( musica == NULL ) {
			printf ("Musica no cargada: %s\n", Mix_GetError());
		}
		Mix_VolumeChunk (musica, volumen);
	} else {
		musica = Mix_LoadWAV (archivo);
		if ( musica == NULL ) {
			printf ("El sonido no ha podido ser cargado: %s\n", Mix_GetError());
		}
		Mix_VolumeChunk (musica, volumen);
	}
	
	numEsquemas = 0;
	ciclo = 0;
	canalAudio = 0;
	
	auxEsquema.cicloIni = 0;
	auxEsquema.cicloFin = 0;
	auxEsquema.tiempoFadeIn = 0;
	auxEsquema.tiempoFadeOut = 0;
	auxEsquema.loops = 0;
}

void Sonido::setEsquema (int cicloIni, int cicloFin, int tiempoFadeIn, int tiempoFadeOut, int loops) {
	
	auxEsquema.cicloIni = cicloIni;
	auxEsquema.cicloFin = cicloFin;
	auxEsquema.tiempoFadeIn = tiempoFadeIn;
	auxEsquema.tiempoFadeOut = tiempoFadeOut;
	auxEsquema.loops = loops;
	
	esquema.push_back(auxEsquema);
	
	numEsquemas++;
}

bool Sonido::play (int i) {
	if (esquema[i].cicloIni == ciclo) {
		return 1;
	}
	return 0;
}

bool Sonido::stop (int i) {
	if (esquema[i].cicloFin == ciclo) {
		return 1;
	}
	return 0;
}

void Sonido::activar () {
	int i;
	for (i=0; i<esquema.size(); i++) {
		if (play(i)) {
			if (esquema[i].tiempoFadeIn > 0) {
				Mix_FadeInChannel (canalAudio, musica, esquema[i].loops, esquema[i].tiempoFadeIn);
			} else {
				Mix_PlayChannel (canalAudio, musica, esquema[i].loops);
			}
		}
		if (stop(i)) {
			if (esquema[i].tiempoFadeOut > 0) {
				Mix_FadeOutChannel(canalAudio, esquema[i].tiempoFadeOut);
			} else {
				Mix_HaltChannel(canalAudio);
			}
		}
	}
	ciclo++;
}

void Sonido::unLoad () {
	Mix_FreeChunk(musica);
}


/*****************************
**
** CLASE FOREGROUND
**
******************************/

ObjetoForeground::ObjetoForeground(SDL_Surface *screen, int anc, int alt) : Objeto() {

	ancho = anc;
	alto = alt;
	
	crearForeground(screen, 0, 0, 0, "", 0, 0, 0);
	
}

void ObjetoForeground::crearForeground (SDL_Surface *screen, int r, int g, int b, char imagen[127], int posicionFondo, int conAlpha, bool fondoTransparente) {
	
	FrameObjeto frFG;
	
	// si no hago esto no salen los ZOOM en negro
	if (r<10) { r = 10; }
	if (g<10) { g = 10; }
	if (b<10) { b = 10; }
	
	colorFondo = SDL_MapRGB(screen->format, r, g, b);
	
	// fondo
	SDL_Surface *surface;

	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, ancho, alto, 16, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, 0);
	if ( surface == NULL ) {
		printf ("El surface no se ha creado\n");
	}
	SDL_FillRect(surface, NULL, colorFondo);
	
	if (fondoTransparente == 1) {
		SDL_SetColorKey(surface, SDL_SRCCOLORKEY, colorFondo);
	}
	
	if (strcmp(imagen, "") != 0) {
		SDL_Surface *img;
		SDL_Rect rect;
		img = IMG_Load (imagen);
		switch(posicionFondo) {
			case BG_TOP_LEFT:
				rect.x = 0;
				rect.y = 0;
			break;
			case BG_TOP_RIGHT:
				rect.x = ancho - img->w;
				rect.y = 0;
			break;
			case BG_BOTTOM_LEFT:
				rect.x = 0;
				rect.y = alto - img->h;
			break;
			case BG_BOTTOM_RIGHT:
				rect.x = ancho - img->w;
				rect.y = alto - img->h;
			break;
			case BG_CENTER:
				rect.x = ancho/2 - img->w/2;
				rect.y = alto/2 - img->h/2;
			break;
		}
		
		SDL_BlitSurface(img, NULL, surface, &rect);
		
	}
	
	
	frFG.loadSurface(surface);
	modificarFrameN(frFG, 0);
	
	setX(0);
	setY(0);
	
	
}


/*****************************
**
** CLASE ANIMACION
**
******************************/

SDL_Animation::SDL_Animation(SDL_Surface *srf_screen, int anc, int alt) {
	tipoTrabajo = EXPLOTACION;
	numeroElementos = 0;
	numeroForeground = 0;
	numeroSonidos = 0;
	numeroFondos = 0;
	ciclo = 0;
	empezarEnCiclo = 0;
	ancho = anc;
	alto = alt;
	imagenFondoActual = -1;
	cicloFin = -1;
	screen = srf_screen;
	strcpy(fuenteTTFParaDevelop, "");
	mostrarCartel = 1;
	pausa = 0;
	forzarFinAnimacion = 0;
	salirConTodasLasTeclas = true;
	
	// iniciamos ttf
	
	if (TTF_Init() < 0) {
		printf ("No se puede iniciar SDL_TTF: %s\n", SDL_GetError());
	}

	// iniciamos sonido	
	/*if (Mix_OpenAudio (44100, AUDIO_S16, 2, 4096) < 0) {
		printf ("No se puede iniciar SDL_MIXER: %s\n", SDL_GetError());
	}
	Mix_AllocateChannels(16); // numeros de canales simultáneos de audio
	*/
	colorFondo = SDL_MapRGB(screen->format, 0, 0, 0);
	rectFondo.x = 0;
	rectFondo.y = 0;
	rectFondo.w = ancho;
	rectFondo.h = alto;	
	
	// fondo desarrollo
	Uint32 rmask, gmask, bmask, amask;
	SDL_Surface *surface;
	rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
	FrameObjeto frFondoDesarrollo;
	surface = SDL_CreateRGBSurface(SDL_SWSURFACE, 110, 60, 8, rmask, gmask, bmask, amask);
	if ( surface == NULL ) {
		printf ("El surface no se ha creado\n");
	}
	frFondoDesarrollo.loadSurface(surface);
	fondoDesarrollo = new Objeto();
	fondoDesarrollo->setX(7);
	fondoDesarrollo->setY(7);
	fondoDesarrollo->setAlpha(100);
	fondoDesarrollo->addFrame(frFondoDesarrollo);

}


void SDL_Animation::cambiarCicloAnimacion (int c) {
	int i;
	ciclo = c;
	
	for (i=0; i<numeroElementos; i++) {
		elementos[i]->setCiclo(c);
	}
	
}

void SDL_Animation::loadXML (char archXML[64], char archRAR[27]) {
	strcpy(archivoComprimido, archRAR);
	parseXML(this, archXML);
}

void SDL_Animation::loadBuffer (string b, char archRAR[27]) {
	strcpy(archivoComprimido, archRAR);
	parseBuffer(this, b);
}

void SDL_Animation::crearLoading(char imagen[127], int conAlpha) {

	FrameObjeto frElemento;
	loading = new Objeto();
	if (conAlpha==1) {
		loading->setImagen_ConAlpha(1);
		frElemento.loadAlpha(imagen);
	} else {
		frElemento.load(imagen);		
	}
	loading->addFrame(frElemento);
	loading->setX(ancho/2);
	loading->setY(alto/2);
	loading->setAlign(ALIGN_CENTER);
	loading->setVAlign(VALIGN_CENTER);
	loading->dibujar(screen);
	SDL_Flip (screen);
	
}


void SDL_Animation::crearElemento(char nombre[15], char imagen[127], int conAlpha) {

	FrameObjeto frElemento;
	Objeto *el = new Objeto();
	el->setNombre(nombre);
	if (conAlpha==1) {
		el->setImagen_ConAlpha(1);
		frElemento.loadAlpha(imagen);
	} else {
		frElemento.load(imagen);		
	}
	el->addFrame(frElemento);
	elementos.push_back(el);
	numeroElementos++;
	
}


void SDL_Animation::crearElementoAnimacion(char nombre[15], int numElementos, int ciclosEntreFrames, int rotacion) {
	
	Objeto *el = new Objeto(numElementos);
	el->setNombre(nombre);
	el->setCiclosEntreFrames(ciclosEntreFrames);
	el->setTipoRotacion(rotacion);
	elementos.push_back(el);
	numeroElementos++;

}

void SDL_Animation::crearElementoAnimacion(char nombre[15], int numElementos, int ciclosEntreFrames, int rotacion, char imagen[127], char ext[5], int conAlpha) {
	
	int i;
	char tmp[127];
	FrameObjeto frElemento;
	
	Objeto *el = new Objeto(numElementos);
	el->setNombre(nombre);
	el->setCiclosEntreFrames(ciclosEntreFrames);
	el->setTipoRotacion(rotacion);
	
	for (i=0; i<numElementos; i++) {
		sprintf (tmp, "%s%d.%s", imagen, i, ext);
			if (conAlpha==1) {
				frElemento.loadAlpha(tmp);
			} else {
				frElemento.load(tmp);
			}
		el->addFrame(frElemento);
	}
	elementos.push_back(el);
	numeroElementos++;
	
}


SDL_Surface* SDL_Animation::surfaceLinea (char fuenteTTF[127], int tamTexto, char txt[127], int fondoTranparente, int fgR, int fgG, int fgB, int bgR, int bgG, int bgB) {
	char tmpf[128];
	TTF_Font *fuente;
	SDL_Color fgcolor, bgcolor;
	SDL_Surface *ttext;

	sprintf (tmpf, "%s", fuenteTTF);
	fuente = TTF_OpenFont(tmpf, tamTexto);
	
	fgcolor.r = fgR;
	fgcolor.g = fgG;
	fgcolor.b = fgB;
	bgcolor.r = bgR;
	bgcolor.g = bgG;
	bgcolor.b = bgB;

	//ttext = TTF_RenderText_Shaded (fuente, txt, fgcolor, bgcolor);
	ttext = TTF_RenderUTF8_Shaded (fuente, txt, fgcolor, bgcolor);

	if (fondoTranparente==1) {
		SDL_SetColorKey ( ttext, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB (ttext->format, bgR,bgG,bgB) );
	}
	

	return ttext;

}

void SDL_Animation::crearTexto(char nombre[15], char texto[512], int tamTexto, char fuenteTTF[127], int text_align, int fondoTranparente, int separacionInterlineal, int x, int y, int fgR, int fgG, int fgB, int bgR, int bgG, int bgB) {

	if (strcmp(texto, "") != 0) {
		SDL_Surface* sfc_linea; // contiene una línea del párrafo
		SDL_Surface* parrafo; // contiene el parrafo completo
		SDL_Rect dest;
		char *linea;
		char texto_cpy[512];
		strcpy(texto_cpy, texto);
		// contar el numero de líneas y recoger el alto de una línea
		int altoLinea, anchoLinea, numLineas;
		linea = strtok( texto, "|" );
		sfc_linea = surfaceLinea(fuenteTTF, tamTexto, linea, fondoTranparente, fgR, fgG, fgB, bgR, bgG, bgB);
		altoLinea = sfc_linea->h;
		anchoLinea = sfc_linea->w;
		int cont = 1;
		while( (linea = strtok( NULL, "|" )) != NULL ) {
			sfc_linea = surfaceLinea(fuenteTTF, tamTexto, linea, fondoTranparente, fgR, fgG, fgB, bgR, bgG, bgB);
			if (anchoLinea<sfc_linea->w) {
				anchoLinea = sfc_linea->w;
			}
			cont++;
		}
		numLineas = cont;
		
		if (numLineas > 1) {
			Uint32 rmask, gmask, bmask, amask;
			#if SDL_BYTEORDER == SDL_BIG_ENDIAN
				rmask = 0xff000000;
				gmask = 0x00ff0000;
				bmask = 0x0000ff00;
				amask = 0x000000ff;
			#else
				rmask = 0x000000ff;
				gmask = 0x0000ff00;
				bmask = 0x00ff0000;
				amask = 0xff000000;
			#endif
		
			parrafo = SDL_CreateRGBSurface(SDL_SWSURFACE, anchoLinea, (sfc_linea->h*numLineas)+(separacionInterlineal*numLineas), 32,
										   rmask, gmask, bmask, amask);
			int colorkey = SDL_MapRGB(parrafo->format, bgR, bgG, bgB);
			SDL_FillRect(parrafo, NULL, colorkey);
			SDL_SetColorKey(parrafo, SDL_SRCCOLORKEY, colorkey);
			linea = strtok( texto_cpy, "|" );
			sfc_linea = surfaceLinea(fuenteTTF, tamTexto, linea, fondoTranparente, fgR, fgG, fgB, bgR, bgG, bgB);
			switch(text_align) {
				case 1:
					dest.x = (anchoLinea/2) - (sfc_linea->w/2);
				break;
				case 2:
					dest.x = anchoLinea - sfc_linea->w;
				break;
				default:
					dest.x = 0;
				break;
			}

			dest.y = 0;
			SDL_BlitSurface (sfc_linea, NULL, parrafo, &dest);
			cont = 0;
			while( (linea = strtok( NULL, "|" )) != NULL ) {
				sfc_linea = surfaceLinea(fuenteTTF, tamTexto, linea, fondoTranparente, fgR, fgG, fgB, bgR, bgG, bgB);
				cont++;
				switch(text_align) {
					case 1:
						dest.x = (anchoLinea/2) - (sfc_linea->w/2);
					break;
					case 2:
						dest.x = anchoLinea - sfc_linea->w;
					break;
					default:
						dest.x = 0;
					break;
				}
				dest.y = (altoLinea*cont) + (separacionInterlineal*cont);
				SDL_BlitSurface (sfc_linea, NULL, parrafo, &dest);
			}
			
			SDL_FreeSurface (sfc_linea);
		}
		
		// creamos el elemento
		// si el nombre es "text-stop", guardamos en el elemento textStop
		FrameObjeto frElemento;
		if (strcmp(nombre, "text-stop") == 0) {
			textStop = new Objeto();
			textStop->setNombre(nombre);
			textStop->setX(x);
			textStop->setY(y);
			textStop->setColoresFondoEnAlpha(bgR, bgG, bgB);
			if (numLineas > 1) {
				frElemento.loadSurface(parrafo);
			} else {
				frElemento.loadSurface(sfc_linea);
			}
			textStop->addFrame(frElemento);
		} else {
			Objeto *el = new Objeto();
			el->setNombre(nombre);
			el->setColoresFondoEnAlpha(bgR, bgG, bgB);
			if (numLineas > 1) {
				frElemento.loadSurface(parrafo);
			} else {
				frElemento.loadSurface(sfc_linea);
			}
			el->addFrame(frElemento);
			elementos.push_back(el);
			numeroElementos++;
		}
		
	}	else {
		printf("El parrafo del elemento %s no puede estar vacío\n", nombre);
	}
}

void SDL_Animation::addImagenElementoAnimacion (char nombre[15], char imagen[127], int conAlpha) {
	int pos = posicionNombreObjeto(nombre);
	FrameObjeto frElemento;
	
	if (pos != -1) {
		if (conAlpha==1) {
			frElemento.loadAlpha(imagen);
			elementos[pos]->setImagen_ConAlpha(1);
		} else {
			frElemento.load(imagen);		
		}
		elementos[pos]->addFrame(frElemento);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
}

int SDL_Animation::posicionNombreObjeto (char nombre[15]) {
	int i;
	
	for (i=0; i<elementos.size(); i++) {
		if (strcmp(elementos[i]->getNombre(), nombre)==0) {
			return i;
		}
	}

	return -1;
	
}


void SDL_Animation::addMovimiento (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX_Fin, posY_Fin, desaparecerAlFinal, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addPosicion (char nombre[15], int cicloIni, int posX, int posY) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloIni+2, posX, posY, posX, posY, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addMovimiento_FadeIn (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeIn, int opacidad_begin, int opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX_Fin, posY_Fin, desaparecerAlFinal, tiempoFadeIn, opacidad_begin, opacidad_end, 0, 0, 0, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addMovimiento_FadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeOut, int opacidad_begin, int opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX_Fin, posY_Fin, desaparecerAlFinal, 0, 0, 0, tiempoFadeOut, opacidad_begin, opacidad_end, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addMovimiento_FadeIn_FadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int posX_Fin, int posY_Fin, int desaparecerAlFinal, int tiempoFadeIn, int tiempoFadeOut, int fadein_opacidad_begin, int fadein_opacidad_end, int fadeout_opacidad_begin, int fadeout_opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX_Fin, posY_Fin, desaparecerAlFinal, tiempoFadeIn, fadein_opacidad_begin, fadein_opacidad_end, tiempoFadeOut, fadeout_opacidad_begin, fadeout_opacidad_end, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addFadeIn (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal, int tiempoFadeIn, int opacidad_begin, int opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX, posY, desaparecerAlFinal, tiempoFadeIn, opacidad_begin, opacidad_end, 0, 0, 0, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addFadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal, int tiempoFadeOut, int opacidad_begin, int opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX, posY, desaparecerAlFinal, 0, 0, 0, tiempoFadeOut, opacidad_begin, opacidad_end, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addFadeIn_FadeOut (char nombre[15], int cicloIni, int cicloFin, int posX, int posY, int desaparecerAlFinal, int tiempoFadeIn, int tiempoFadeOut, int fadein_opacidad_begin, int fadein_opacidad_end, int fadeout_opacidad_begin, int fadeout_opacidad_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setMovimiento(cicloIni, cicloFin, posX, posY, posX, posY, desaparecerAlFinal, tiempoFadeIn, fadein_opacidad_begin, fadein_opacidad_end, tiempoFadeOut, fadeout_opacidad_begin, fadeout_opacidad_end, 0, 0, 0, 0);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}


void SDL_Animation::addCortinillaIn_Vertical (char nombre[15],int tiempoCortinillaIn, bool UpDown) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setCortinillaVIn_EnMovimientoActual(tiempoCortinillaIn, UpDown);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addCortinillaIn_Horizontal (char nombre[15],int tiempoCortinillaIn, bool LeftRight) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setCortinillaHIn_EnMovimientoActual(tiempoCortinillaIn, LeftRight);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addCortinillaOut_Vertical (char nombre[15],int tiempoCortinillaOut, bool UpDown) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setCortinillaVOut_EnMovimientoActual(tiempoCortinillaOut, UpDown);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addCortinillaOut_Horizontal (char nombre[15],int tiempoCortinillaOut, bool LeftRight) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setCortinillaHOut_EnMovimientoActual(tiempoCortinillaOut, LeftRight);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addZoomIn (char nombre[15],int tiempozoom, double factor_begin, double factor_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setZoomIn_EnMovimientoActual(tiempozoom, factor_begin, factor_end);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addZoomOut (char nombre[15],int tiempozoom, double factor_begin, double factor_end) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setZoomOut_EnMovimientoActual(tiempozoom, factor_begin, factor_end);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addRotateInicial (char nombre[15], int grados) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setRotate(grados);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addZoomInicial (char nombre[15], double factor) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setZoom(factor);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addRotateIn (char nombre[15],int tiemporotar, int angle_begin, int angle_end, int factor) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setRotateIn_EnMovimientoActual(tiemporotar, angle_begin, angle_end, factor);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addRotateOut (char nombre[15],int tiemporotar, int angle_begin, int angle_end, int factor) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setRotateOut_EnMovimientoActual(tiemporotar, angle_begin, angle_end, factor);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addAlineacion (char nombre[15],int align, int valign) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setAlign(align);
		elementos[pos]->setVAlign(valign);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addColoresFondoEnAlpha (char nombre[15],int r, int g, int b) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setColoresFondoEnAlpha (r, g, b);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::addControlAnimation(char nombre[15], ControlAnimation ca) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->addControlAnimation (ca);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::imprimirValoresObjeto (char nombre[15]) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		printf("ELEMENTO: %s\n", nombre);
		printf("Ciclo: %d\n", elementos[pos]->getCiclo());
		printf("Ciclo comienzo: %d\n", elementos[pos]->getCicloNacimiento());
		printf("Ciclo fin: %d\n", elementos[pos]->getCicloMuerte());
		printf("Posicion X: %f\n", elementos[pos]->getX());
		printf("Posicion Y: %f\n", elementos[pos]->getY());
		printf("Activo: %d\n", elementos[pos]->getActivo());
		printf("Ancho: %d\n", elementos[pos]->getAncho());
		printf("Alto: %d\n", elementos[pos]->getAlto());
		printf("Imagen con alpha: %d\n", elementos[pos]->getImagen_ConAlpha());
		printf("Ciclos entre frames (solo animacion): %d\n", elementos[pos]->getCiclosEntreFrames());
		printf("Fade In: %d\n", elementos[pos]->getFadeIn());
		printf("Fade Out: %d\n", elementos[pos]->getFadeOut());
		printf("Cortinilla Hor. In: %d\n", elementos[pos]->getCortinillaHIn());
		printf("Cortinilla Ver. In: %d\n", elementos[pos]->getCortinillaVIn());
		printf("Cortinilla Hor. Out: %d\n", elementos[pos]->getCortinillaHOut());
		printf("Cortinilla Ver. Out: %d\n", elementos[pos]->getCortinillaVOut());
		printf("Zoom In: %d\n", elementos[pos]->getZoomIn());
		printf("Zoom In - Factor comienzo: %f\n", elementos[pos]->getZoomInFactorBegin());
		printf("Zoom In - Factor final: %f\n", elementos[pos]->getZoomInFactorEnd());
		printf("Zoom Out: %d\n", elementos[pos]->getZoomOut());
		printf("Zoom Out - Factor comienzo: %f\n", elementos[pos]->getZoomOutFactorBegin());
		printf("Zoom Out - Factor final: %f\n", elementos[pos]->getZoomOutFactorEnd());
		printf("Rotate In: %d\n", elementos[pos]->getRotateIn());
		printf("Rotate Out: %d\n", elementos[pos]->getRotateOut());
		printf("\n");
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
	
}

void SDL_Animation::imprimirValoresTodosLosObjeto () {
	int i;
	
	printf("\n");
	printf("===============================\n");
	printf("ELEMENTOS EN EL CICLO %d\n", ciclo);
	printf("===============================\n");
	printf("\n");
	for (i=0; i<numeroElementos; i++) {
		imprimirValoresObjeto(elementos[i]->getNombre());
	}
}

void SDL_Animation::mover () {
	int i;
	
	for (i=0; i<numeroElementos; i++) {
		if (elementos[i]->getCicloNacimiento() == ciclo-1) {
			elementos[i]->setActivo(1);
		}
		if (elementos[i]->getCicloMuerte() == ciclo) {
			elementos[i]->setActivo(0);
		}
		elementos[i]->mover();
	}
	
}


void SDL_Animation::addFondo(int ciclo, int r, int g, int b, char imagen[127], int conAlpha, int posicionFondo) { 
	
	Fondo f;
	
	f.cycle = ciclo; 
	f.red = r; 
	f.green = g; 
	f.blue = b; 

	
	if (strcmp(imagen, "") != 0) {
		f.position = posicionFondo;
		FrameObjeto frFondo;
		f.fondo = new Objeto();
		if (conAlpha==1) {
			f.fondo->setImagen_ConAlpha(1);
			frFondo.loadAlpha(imagen);
		} else {
			frFondo.load(imagen);		
		}
		f.fondo->addFrame(frFondo);
		f.fondo->setX(0);
		f.fondo->setY(0);
	} else {
		f.position = -1;
	}

	fondos.push_back(f);
	
	numeroFondos++; 
	
}
	
	
void SDL_Animation::addForeground (int cicloIni, int cicloFin, int r, int g, int b, char imagen[127], int conAlpha, int posicionFondo, bool fondoTransparente) { 
	
	Foreground f;
	
	f.fg = new ObjetoForeground(screen, ancho, alto);
	
	f.fg->crearForeground(screen, r, g, b, imagen, posicionFondo, conAlpha, fondoTransparente);
	f.fg->setX(ancho/2);
	f.fg->setY(alto/2);
	f.fg->setAlign(ALIGN_CENTER);
	f.fg->setVAlign(VALIGN_CENTER);
	f.fg->setImagen_ConAlpha(conAlpha);
	f.cycle_begin = cicloIni;
	f.cycle_end = cicloFin;
	
	fg.push_back(f);
	
	numeroForeground++; 
	
}

void SDL_Animation::addForeground_FadeIn(int tiempoFade) {
	fg[numeroForeground-1].fg->setFadeIn(tiempoFade, 0, 255);
}

void SDL_Animation::addForeground_FadeOut(int tiempoFade) {
	fg[numeroForeground-1].tiempoFadeOut = tiempoFade;
}

void SDL_Animation::addForeground_ZoomIn(int tiempozoom, double factor_begin, double factor_end) {
	fg[numeroForeground-1].fg->setZoomIn(tiempozoom);
	fg[numeroForeground-1].fg->setZoomInFactorBegin(factor_begin);
	fg[numeroForeground-1].fg->setZoomInFactorEnd(factor_end);
}

void SDL_Animation::addForeground_ZoomOut(int tiempozoom, double factor_begin, double factor_end) {
	fg[numeroForeground-1].tiempo_zoom_out = tiempozoom;
	fg[numeroForeground-1].tiempo_zoom_out_factor_begin = factor_begin;
	fg[numeroForeground-1].tiempo_zoom_out_factor_end = factor_end;
}

void SDL_Animation::addForeground_CurtainVIn(int tiempoCurtain) {
	fg[numeroForeground-1].fg->setCortinillaVIn(tiempoCurtain);
}

void SDL_Animation::addForeground_CurtainHIn(int tiempoCurtain) {
	fg[numeroForeground-1].fg->setCortinillaHIn(tiempoCurtain);	
}

void SDL_Animation::addForeground_CurtainVOut(int tiempoCurtain) {
	fg[numeroForeground-1].tiempo_cortinillaV_out = tiempoCurtain;
}

void SDL_Animation::addForeground_CurtainHOut(int tiempoCurtain) {
	fg[numeroForeground-1].tiempo_cortinillaH_out = tiempoCurtain;
}

void SDL_Animation::addDesaparicion (char nombre[15], int ciclo) {
	int pos = posicionNombreObjeto(nombre);
	if (pos != -1) {
		elementos[pos]->setCicloMuerte(ciclo);
	} else {
		printf("El nombre %s no corresponde a ningún elemento\n", nombre);
	}
}

void SDL_Animation::dibujar () {
	int i;
	// PONER FONDO
	SDL_FillRect(screen, &rectFondo, colorFondo);
	if (imagenFondoActual != -1) {
		if (fondos[imagenFondoActual].position == BG_MOSAIC) {
			// hacer mosaico
			int auxAlto = 0, auxAncho = 0;
			fondos[imagenFondoActual].fondo->setX(0);
			fondos[imagenFondoActual].fondo->setY(0);
			while (auxAlto < alto) {
				while (auxAncho < ancho) {
					fondos[imagenFondoActual].fondo->setX(auxAncho);
					fondos[imagenFondoActual].fondo->setY(auxAlto);
					fondos[imagenFondoActual].fondo->dibujar(screen);
					auxAncho += fondos[imagenFondoActual].fondo->getAncho();
				}
				auxAncho = 0;
				auxAlto += fondos[imagenFondoActual].fondo->getAlto();
			}
		} else {
			fondos[imagenFondoActual].fondo->dibujar(screen);
		}
	}
	
	// DIBUJAR ELEMENTOS
	for (i=0; i<numeroElementos; i++) {
		if (elementos[i]->getActivo() == 1) {
			elementos[i]->dibujar(screen);
		}
	}
	
	// PONER FOREGROUND

	for (i=0; i<fg.size(); i++) {
		// comprobar los OUT
		// fade
		if (fg[i].cycle_end - fg[i].tiempoFadeOut == ciclo) {
			fg[i].fg->setFadeOut(fg[i].tiempoFadeOut, 255, 0);
		}
		// zoom
		if (fg[i].cycle_end - fg[i].tiempo_zoom_out == ciclo) {
			fg[i].fg->setZoomOut(fg[i].tiempo_zoom_out);
			fg[i].fg->setZoomOutFactorBegin(fg[i].tiempo_zoom_out_factor_begin);
			fg[i].fg->setZoomOutFactorEnd(fg[i].tiempo_zoom_out_factor_end);
		}
		// cortinillas
		if (fg[i].cycle_end - fg[i].tiempo_cortinillaH_out == ciclo) {
			fg[i].fg->setCortinillaHOut(fg[i].tiempo_cortinillaH_out);
		}
		if (fg[i].cycle_end - fg[i].tiempo_cortinillaV_out == ciclo) {
			fg[i].fg->setCortinillaVOut(fg[i].tiempo_cortinillaV_out);
		}
		// dibujar si existe el foreground en el ciclo
		if ( (fg[i].cycle_begin <= ciclo) && (fg[i].cycle_end >= ciclo) ) {
			fg[i].fg->dibujar(screen);
		}
	}

}

void SDL_Animation::comprobarFondo () {
	int i;
	for (i=0; i<numeroFondos; i++) {
		if (fondos[i].cycle <= ciclo) {
			colorFondo = SDL_MapRGB(screen->format, fondos[i].red, fondos[i].green, fondos[i].blue);
			if (fondos[i].position == -1) {
				imagenFondoActual = -1;
			} else {
				imagenFondoActual = i;
				switch(fondos[i].position) {
					case BG_TOP_LEFT:
						fondos[imagenFondoActual].fondo->setX(0);
						fondos[imagenFondoActual].fondo->setY(0);
					break;
					case BG_TOP_RIGHT:
						fondos[imagenFondoActual].fondo->setX(ancho - fondos[imagenFondoActual].fondo->sprite->img->w);
						fondos[imagenFondoActual].fondo->setY(0);
					break;
					case BG_BOTTOM_LEFT:
						fondos[imagenFondoActual].fondo->setX(0);
						fondos[imagenFondoActual].fondo->setY(alto - fondos[imagenFondoActual].fondo->sprite->img->h);
					break;
					case BG_BOTTOM_RIGHT:
						fondos[imagenFondoActual].fondo->setX(ancho - fondos[imagenFondoActual].fondo->sprite->img->w);
						fondos[imagenFondoActual].fondo->setY(alto - fondos[imagenFondoActual].fondo->sprite->img->h);
					break;
					case BG_CENTER:
						fondos[imagenFondoActual].fondo->setX(ancho/2 - fondos[imagenFondoActual].fondo->sprite->img->w/2);
						fondos[imagenFondoActual].fondo->setY(alto/2 - fondos[imagenFondoActual].fondo->sprite->img->h/2);
					break;
				}
			}
		}
	}
	
}

bool SDL_Animation::hacerStop () {
	int i;
	SDL_Event event;
	
	for (i=0; i<stop.size(); i++) {
		if (stop[i] == ciclo) {
			while (SDL_PollEvent(&event)) {
				if (event.type == SDL_KEYDOWN) {
					if (event.key.keysym.sym == SDLK_ESCAPE) { 
						forzarFinAnimacion = 1;
					}
					return 0;
				}
			}
			return 1;
		}
	}
	return 0;
}

void SDL_Animation::comprobarTeclas () {

	while (SDL_PollEvent(&event)) {
		if ( (event.type == SDL_KEYDOWN) || (event.type == SDL_JOYBUTTONDOWN) ) {
			if (tipoTrabajo == DESARROLLO) {
				if (event.key.keysym.sym == SDLK_c) { 
					if (mostrarCartel == 1) {
						mostrarCartel = 0;
					} else {
						mostrarCartel = 1;
					}
				} else if (event.key.keysym.sym == SDLK_p) { 
					if (pausa == 1) {
						pausa = 0;
					} else {
						imprimirValoresTodosLosObjeto ();
						pausa = 1;
					}
				} else {
					forzarFinAnimacion = 1;	
				}
			} else {
				if (salirConTodasLasTeclas) {
					forzarFinAnimacion = 1;
				} else {
					if ( (event.key.keysym.sym == SDLK_ESCAPE) 
						|| ( (event.jbutton.button == 0) && (event.type == SDL_JOYBUTTONDOWN) ) ) { 
						forzarFinAnimacion = 1;
					}
				}
			}
		}
	}


	
}

bool SDL_Animation::DrawAnimation () {
	if (ciclo == 0) {
		if (ciclo != empezarEnCiclo) {
			cambiarCicloAnimacion(empezarEnCiclo);
		}
	}
	

	if (pausa == 0) {
		if (hacerStop ()) {
			textStop->dibujar(screen);
			return 0;
		}
		activarMusica();
		comprobarFondo ();
		mover();
		dibujar();
		ciclo++;
		
		if ( (tipoTrabajo == DESARROLLO) && (mostrarCartel==1) ) {
			int clicos_mostrar = ciclo - ciclo % 10;
			char tmp[10];
			fondoDesarrollo->dibujar(screen);
			sprintf (tmp, "ciclo: %d", clicos_mostrar);
			escribirTexto(screen, ALIGN_LEFT, VALIGN_TOP, 12, tmp, 255, 255, 255, 0, 0, 0, 10, 10);
			sprintf (tmp, "objetos: %d", numeroElementos);
			escribirTexto(screen, ALIGN_LEFT, VALIGN_TOP, 12, tmp, 255, 255, 255, 0, 0, 0, 10, 27);
			sprintf (tmp, "sonidos: %d", numeroSonidos);
			escribirTexto(screen, ALIGN_LEFT, VALIGN_TOP, 12, tmp, 255, 255, 255, 0, 0, 0, 10, 44);
		}
	}
	
	comprobarTeclas();
	
	
	if ( (cicloFin == ciclo) || (forzarFinAnimacion==1) ) {
		unLoad();
		return 1;
	} else {
		return 0;
	}
}


void SDL_Animation::escribirTexto (SDL_Surface *sc, int align, int valign, int size, char txt[63], int fgR, int fgG, int fgB, int bgR, int bgG, int bgB, int x, int y) {
	if (strcmp(txt, "") != 0) {
		//char tmpf[50];
		TTF_Font *fuente;
		SDL_Color fgcolor, bgcolor;
		SDL_Surface *ttext;
		SDL_Rect rect_dest, rect_src;
	
		//sprintf (tmpf, fuenteTTFParaDevelop);
		fuente = TTF_OpenFont(fuenteTTFParaDevelop, size);
		
		fgcolor.r = fgR;
		fgcolor.g = fgG;
		fgcolor.b = fgB;
		bgcolor.r = bgR;
		bgcolor.g = bgG;
		bgcolor.b = bgB;
	
		//ttext = TTF_RenderText_Shaded (fuente, txt, fgcolor, bgcolor);
		ttext = TTF_RenderUTF8_Shaded (fuente, txt, fgcolor, bgcolor);
		
		switch (align)
		{
			case  1:
				rect_dest.x = x - ttext->w/2;
				break;
			case  2:
				rect_dest.x = x - ttext->w;
				break;
			default:
				rect_dest.x = x;
				break;
		}
		
		switch (valign)
		{
			case  1:
				rect_dest.y = y - ttext->h/2;
				break;
			case  2:
				rect_dest.y = y - ttext->h;
				break;
			default:
				rect_dest.y = y;
				break;
		}
		rect_src.x = 0;
		rect_src.y = 0;
		rect_src.w = ttext->w;
	
		rect_src.h = ttext->h;
		SDL_SetColorKey ( ttext, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB (ttext->format, bgR,bgG,bgB) );
		
		SDL_BlitSurface ( ttext, NULL, sc, &rect_dest );
		
		TTF_CloseFont(fuente);
		SDL_FreeSurface (ttext);
	
	}	
}


int SDL_Animation::posicionNombreSonido (char nombre[27]) {
	int i;
	
	for (i=0; i<sonidos.size(); i++) {
		if (strcmp(sonidos[i]->getNombre(), nombre)==0) {
			return i;
		}
	}
	
	return -1;
	
}

void SDL_Animation::crearSonido (char nombre[15], char archivo[127], int volumen) {
	Sonido *son = new Sonido(archivo, volumen);
	son->setNombre(nombre);
	son->setCanalAudio(numeroSonidos);
	sonidos.push_back(son);
	numeroSonidos++;
}

void SDL_Animation::addEsquema (char nombre[15], int cicloIni, int cicloFin, int tiempoFadeIn, int tiempoFadeOut, int loops) {
	int pos = posicionNombreSonido(nombre);
	if (pos != -1) {
		sonidos[pos]->setEsquema(cicloIni, cicloFin, tiempoFadeIn, tiempoFadeOut, loops);
	} else {
		printf("El nombre %s no corresponde a ningún sonido\n", nombre);
	}
	
}

void SDL_Animation::activarMusica () {
	int i;
	
	for (i=0; i<sonidos.size(); i++) {
		sonidos[i]->activar();
	}
	
}

void SDL_Animation::inVariable (string nombre, string valor) {

	variables[nombre]= valor;
	
}

string SDL_Animation::outVariable (string nombre) {

	map<string, string>::iterator var;
	
	for (var = variables.begin(); var != variables.end(); ++var) {
		if ( nombre ==  (*var).first) {
			return (*var).second;
		}
	}
	
	return "";
	
}

void SDL_Animation::imprimirVariables () {

	map<string, string>::iterator var;
	
	for (var = variables.begin(); var != variables.end(); ++var) {
		cout << (*var).first << " : " << (*var).second << endl;
	}
	
}

char* SDL_Animation::convertirRuta (string cadena) {
	char ruta[80], *auxRuta;
	char auxcadena[80];
	
	strcpy(ruta, "");
	strcpy(auxcadena, cadena.data());
	
	auxRuta = strtok( auxcadena, "$" );
	if (strcmp(outVariable(auxRuta).data(), "") != 0) {
		sprintf(ruta, "%s%s", ruta, outVariable(auxRuta).data());
	} else {
		sprintf(ruta, "%s%s", ruta, auxRuta);
	}
	while( (auxRuta = strtok( NULL, "$" )) != NULL )  { 
		if (strcmp(outVariable(auxRuta).data(), "") != 0) {
			sprintf(ruta, "%s%s", ruta, outVariable(auxRuta).data());
		} else {
			sprintf(ruta, "%s%s", ruta, auxRuta);
		}
	}
	
	//printf("%s\n", ruta);
	
	return ruta;
	
}

void SDL_Animation::unLoad () {

	int i;
	
	for (i=0; i<numeroElementos; i++) {
		elementos[i]->unLoad();
	}
	
	for (i=0; i<numeroSonidos; i++) {
		sonidos[i]->unLoad();
	}
	
	for (i=0; i<numeroFondos; i++) {
		if (fondos[i].position != -1) {
			fondos[i].fondo->unLoad();
		}
	}
	
	fondoDesarrollo->unLoad();
	
}

/*****************************
**
** FUNCIONES PARA TRABAJAR
** CON ARCHIVO XML
**
******************************/

typedef struct
{
	int cycle_in, cycle_out, time_fadein, time_fadeout, loops;			
} XMLSound;

typedef struct
{
	int cycle, red, green, blue;
	char file[64];
	int image_with_alpha;
	int background_align;
} XMLBackground;

typedef struct
{
	int red, green, blue;
	char file[64];
	int image_with_alpha;
	int background_align;
} XMLForeground;

typedef struct
{
	char name[32];
	char value[64];
} XMLVariable;

typedef struct
{
	char file[64];
	int image_with_alpha;
} XMLFrames;


typedef struct
{
	int cycle_begin, cycle_end, coord_begin_x, coord_begin_y, coord_end_x, coord_end_y;
	int fadein, fadeout;
	int fadein_factor_begin, fadein_factor_end;
	int fadeout_factor_begin, fadeout_factor_end;
	int	curtainVIn, curtainVOut, curtainHIn, curtainHOut, disappear_to_end;
	bool curtainVInUpDown;
	bool curtainHInLeftRight;
	bool curtainVOutUpDown;
	bool curtainHOutLeftRight;
	int zoomin, zoomout;
	float zoomin_factor_begin, zoomin_factor_end;
	float zoomout_factor_begin, zoomout_factor_end;
	int rotatein, rotateout;
	int factorRotatein,  factorRotateout;
	int rotatein_angle_begin, rotatein_angle_end;
	int rotateout_angle_begin, rotateout_angle_end;
} XMLMotion;

// variables auxiliares a los cuales se le irán agregando datos hasta que termine
// y se agrege al vector correspodiente

XMLSound auxSonido;
XMLMotion auxMotion;
XMLBackground auxFondo;
XMLVariable auxVariable;
XMLFrames auxFrame;
ControlAnimation auxControlAnimation;

typedef struct
{
	SDL_Animation *anim;
	int numObjects;
	char name[15];
	char type[8];
	int volumen;
	char file[64];
	char text[512];
	int font_size;
	char font_ttf[128];
	int background_transparent;
	int x, y;
	int fgR, fgG, fgB, bgR, bgG, bgB;
	int image_with_alpha;
	int cycles_between_frames;
	int rotation_circle;
	int separate_line;
	int rotateInicial;
	float zoomInicial;
	int numMov;
	vector<XMLMotion> mov;
	int numSon;
	vector<XMLSound> son;
	int numFrames;
	vector<XMLFrames> frames;
	int numFondos;
	vector<XMLBackground> fondos;
	int numVariables;
	vector<XMLVariable> variables;
	vector<ControlAnimation> controlAnimation;
	XMLForeground fg;
	vector<int> stop;
	int numStop;
	int cycle_disappear;
	bool datMov, datLoc, datFrame, datSonido;
	int align, valign, text_align;
} UserData;

void iniciarUserData(UserData *data) {

	strcpy(data->name, "");
	strcpy(data->type, "");
	data->volumen = 0;
	strcpy(data->file, "");
	strcpy(data->text, "");
	data->font_size = 0;
	strcpy(data->font_ttf, "");
	data->background_transparent = 0;
	data->x = 0;
	data->y = 0;
	data->fgR = 0;
	data->fgG = 0;
	data->fgB = 0;
	data->bgR = 0;
	data->bgG = 0;
	data->bgB = 0;
	data->image_with_alpha = 0;
	data->cycles_between_frames=0;
	data->rotation_circle = 0;
	data->separate_line = 0;
	data->fg.red = 0;
	data->fg.green = 0;
	data->fg.blue = 0;
	data->fg.background_align = BG_CENTER;
	strcpy(data->fg.file, "");
	data->fg.image_with_alpha = 0;
	data->zoomInicial = 1;
	data->rotateInicial = 0;
	data->mov.resize(0);
	data->numMov = 0;
	data->son.resize(0);
	data->numSon = 0;

	data->frames.resize(0);
	data->numFrames = 0;

	data->fondos.resize(0);
	data->numFondos = 0;
	
	data->controlAnimation.resize(0);

	data->numVariables = 0;
	data->numStop = 0;
	data->datMov = 0;
	data->datLoc = 0;
	data->datFrame = 0;
	data->datSonido = 0;
	data->cycle_disappear = -1;
	data->align = 0;
	data->valign = 0;
	data->text_align = 0;
}

void iniciarAuxiliares() {
	
	auxMotion.cycle_begin = 0;
	auxMotion.cycle_end = 0;
	auxMotion.coord_begin_x = 0;
	auxMotion.coord_begin_y = 0;
	auxMotion.coord_end_x = 999999;
	auxMotion.coord_end_y = 999999;
	auxMotion.fadein = -1;
	auxMotion.fadein_factor_begin = 0;
	auxMotion.fadein_factor_end = 255;
	auxMotion.fadeout = -1;
	auxMotion.fadeout_factor_begin = 255;
	auxMotion.fadeout_factor_end = 0;
	auxMotion.curtainVIn = -1;
	auxMotion.curtainVOut = -1;
	auxMotion.curtainHIn = -1;
	auxMotion.curtainHOut = -1;
	auxMotion.curtainVInUpDown = 1;
	auxMotion.curtainHInLeftRight = 1;
	auxMotion.curtainVOutUpDown = 1;
	auxMotion.curtainHOutLeftRight = 1;
	auxMotion.zoomin = -1;
	auxMotion.zoomin_factor_begin = -1.0;
	auxMotion.zoomin_factor_end = -1.0;
	auxMotion.zoomout = -1;
	auxMotion.zoomout_factor_begin = -1.0;
	auxMotion.zoomout_factor_end = -1.0;
	auxMotion.rotatein = -1;
	auxMotion.factorRotatein = 1;
	auxMotion.rotatein_angle_begin = 0;
	auxMotion.rotatein_angle_end = 0;
	auxMotion.rotateout = -1;
	auxMotion.factorRotateout = 1;
	auxMotion.rotateout_angle_begin = 0;
	auxMotion.rotateout_angle_end = 0;
	auxMotion.disappear_to_end = 0;
	
	auxSonido.cycle_in = 0;
	auxSonido.cycle_out = 0;
	auxSonido.time_fadein = 0;
	auxSonido.time_fadeout = 0;
	auxSonido.loops = 0;
	
	auxFondo.cycle = -1;
	auxFondo.red = 0;
	auxFondo.green = 0;
	auxFondo.blue = 0;
	auxFondo.background_align = BG_CENTER;
	strcpy(auxFondo.file, "");
	auxFondo.image_with_alpha = 0;
	
	strcpy(auxVariable.name, "");
	strcpy(auxVariable.value, "");
	
	strcpy(auxFrame.file, "");
	auxFrame.image_with_alpha = 0;
	
	auxControlAnimation.isPlay = 0;
	auxControlAnimation.cycle = 0;
	auxControlAnimation.frame = -1;
	
}

static void startXML(void *userData, const char *el, const char **attr) {
  int i;
  UserData* data = (UserData*)userData;

	if (data->datMov == 1 ) { // se está produciendo un movimiento
		if (strcmp(el, "cycle") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "begin") == 0) {
					auxMotion.cycle_begin = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "end") == 0) {
					auxMotion.cycle_end = atoi(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "coord-begin") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "x") == 0) {
					auxMotion.coord_begin_x = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "y") == 0) {
					auxMotion.coord_begin_y = atoi(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "coord-end") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "x") == 0) {
					auxMotion.coord_end_x = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "y") == 0) {
					auxMotion.coord_end_y = atoi(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "fade-in") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.fadein = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "opacity-begin") == 0) {
					auxMotion.fadein_factor_begin = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "opacity-end") == 0) {
					auxMotion.fadein_factor_end = atoi(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "fade-out") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.fadeout = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "opacity-begin") == 0) {
					auxMotion.fadeout_factor_begin = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "opacity-end") == 0) {
					auxMotion.fadeout_factor_end = atoi(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "curtain-in-vertical") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainVIn = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "vertical") == 0) {
					if (strcmp(attr[i+1], "down-up") == 0) {
						auxMotion.curtainVInUpDown = 0;
					}
				}
			}
		}
		if (strcmp(el, "curtain-out-vertical") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainVOut = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "vertical") == 0) {
					if (strcmp(attr[i+1], "down-up") == 0) {
						auxMotion.curtainVOutUpDown = 0;
					}
				}
			}
		}
		if (strcmp(el, "curtain-in") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainVIn = atoi(attr[i+1]);
					auxMotion.curtainHIn = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "vertical") == 0) {
					if (strcmp(attr[i+1], "down-up") == 0) {
						auxMotion.curtainVInUpDown = 0;
					}
				}
				if (strcmp(attr[i], "horizontal") == 0) {
					if (strcmp(attr[i+1], "right-left") == 0) {
						auxMotion.curtainHInLeftRight = 0;
					}
				}
			}
		}
		if (strcmp(el, "curtain-out") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainVOut = atoi(attr[i+1]);
					auxMotion.curtainHOut = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "vertical") == 0) {
					if (strcmp(attr[i+1], "down-up") == 0) {
						auxMotion.curtainVOutUpDown = 0;
					}
				}
				if (strcmp(attr[i], "horizontal") == 0) {
					if (strcmp(attr[i+1], "right-left") == 0) {
						auxMotion.curtainHOutLeftRight = 0;
					}
				}
			}
		}
		if (strcmp(el, "curtain-in-horizontal") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainHIn = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "horizontal") == 0) {
					if (strcmp(attr[i+1], "right-left") == 0) {
						auxMotion.curtainHInLeftRight = 0;
					}
				}
			}
		}
		if (strcmp(el, "curtain-out-horizontal") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.curtainHOut = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "horizontal") == 0) {
					if (strcmp(attr[i+1], "right-left") == 0) {
						auxMotion.curtainHOutLeftRight = 0;
					}
				}
			}
		}
		if (strcmp(el, "zoom-in") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.zoomin = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "factor-begin") == 0) {
					auxMotion.zoomin_factor_begin = atof(attr[i+1]);
				}
				if (strcmp(attr[i], "factor-end") == 0) {
					auxMotion.zoomin_factor_end = atof(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "zoom-out") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.zoomout = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "factor-begin") == 0) {
					auxMotion.zoomout_factor_begin = atof(attr[i+1]);
				}
				if (strcmp(attr[i], "factor-end") == 0) {
					auxMotion.zoomout_factor_end = atof(attr[i+1]);
				}
			}
		}
		if (strcmp(el, "rotate-in") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.rotatein = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "angle-begin") == 0) {
					auxMotion.rotatein_angle_begin = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "angle-end") == 0) {
					auxMotion.rotatein_angle_end = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "rotation") == 0) {
					if (strcmp(attr[i+1], "cw") == 0) {
						auxMotion.factorRotatein = -1;
					} else {
						auxMotion.factorRotatein = 1;
					}
				}
			}
		}
		if (strcmp(el, "rotate-out") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycles") == 0) {
					auxMotion.rotateout = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "angle-begin") == 0) {
					auxMotion.rotateout_angle_begin = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "angle-end") == 0) {
					auxMotion.rotateout_angle_end = atoi(attr[i+1]);
				}
				if (strcmp(attr[i], "rotation") == 0) {
					if (strcmp(attr[i+1], "clockwise") == 0) {
						auxMotion.factorRotateout = -1;
					} else {
						auxMotion.factorRotateout = 1;
					}
				}
			}
		}
		if (strcmp(el, "disappear-to-end") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "value") == 0) {
					if ( strcmp(attr[i+1], "true") == 0 ) {
						auxMotion.disappear_to_end = 1;
					}
				}
			}
		}
	} else if (data->datLoc == 1 ) { // se está produciendo una localización
		if (strcmp(el, "cycle") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "begin") == 0) {
					auxMotion.cycle_begin = atoi(attr[i+1]);
					auxMotion.cycle_end = atoi(attr[i+1])+2;
				}
			}
		}
		if (strcmp(el, "coord-begin") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "x") == 0) {
					auxMotion.coord_begin_x = atoi(attr[i+1]);
					auxMotion.coord_end_x = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "y") == 0) {
					auxMotion.coord_begin_y = atoi(attr[i+1]);
					auxMotion.coord_end_y = atoi(attr[i+1]);
				}
			}
		}
	} else if (data->datFrame == 1 ) { // se está produciendo una animacion de frames
		if (strcmp(el, "frame") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "file") == 0) {
					strcpy(auxFrame.file, data->anim->convertirRuta(attr[i+1]));
				} else if (strcmp(attr[i], "image-with-alpha") == 0) {
					if (strcmp(attr[i+1], "true") == 0) {
						auxFrame.image_with_alpha = 1;
					}
				}
			}
		}
	} else if (data->datSonido == 1 ) { // se está produciendo un sonido
		if (strcmp(el, "sound") == 0) {
			for (i = 0; attr[i]; i += 2) {
				if (strcmp(attr[i], "cycle-in") == 0) {
					auxSonido.cycle_in = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "cycle-out") == 0) {
					auxSonido.cycle_out = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "time-fadein") == 0) {
					auxSonido.time_fadein = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "time-fadeout") == 0) {
					auxSonido.time_fadeout = atoi(attr[i+1]);
				} else if (strcmp(attr[i], "loops") == 0) {
					auxSonido.loops = atoi(attr[i+1]);
				}
			}
		}
	} else { // una marquilla de objeto nuevo
		if (strcmp(el, "object") == 0) {
		  iniciarUserData(data);
		  sprintf(data->name, "object%d", data->numObjects);
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "name") == 0) {
				  sprintf(data->name, "%s", attr[i+1]);
			  }
			  if (strcmp(attr[i], "type") == 0) {
				  strcpy(data->type, attr[i+1]);
			  }
		  }
		} else if (strcmp(el, "control") == 0) {
		  iniciarUserData(data); // tomamos control como si fuese un objeto e inicializamos
		} else if (strcmp(el, "key-quit") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if ( (strcmp(attr[i], "value") == 0) || (strcmp(attr[i+1], "ESCAPE") == 0) ) {
				  data->anim->setSalirConTodasLasTeclas(false);
			  }
		  }
		} else if (strcmp(el, "variable") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "name") == 0) {
				  strcpy(auxVariable.name, attr[i+1]);
			  }
			  if (strcmp(attr[i], "value") == 0) {
				  strcpy(auxVariable.value, attr[i+1]);
			  }
		  }
		} else if (strcmp(el, "work") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "value") == 0) {
				  if (strcmp(attr[i+1], "develop") == 0) {
					  data->anim->setTrabajo(DESARROLLO);
				  }
			  }
			  if (strcmp(attr[i], "font-ttf") == 0) {
				  data->anim->setFuenteTTFParaDevelop(data->anim->convertirRuta(attr[i+1]));
			  }
		  }
		} else if (strcmp(el, "begin-animation") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  data->anim->setEmpezarEnCiclo(atoi(attr[i+1]));
			  }
		  }
		} else if (strcmp(el, "stop") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  data->stop.push_back (atoi(attr[i+1]));
			  }
		  }
		} else if (strcmp(el, "end") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  data->anim->setCicloFin(atoi(attr[i+1]));
			  }
		  }
		} else if (strcmp(el, "load") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "file") == 0) {
				  strcpy(data->file, data->anim->convertirRuta(attr[i+1])); 
			  } else if (strcmp(attr[i], "image-with-alpha") == 0) {
				  if (strcmp(attr[i+1], "true")==0) {
					  data->image_with_alpha = 1;
			  	  }
			  }
		  }
		  data->anim->crearLoading(data->file, data->image_with_alpha);
		} else if (strcmp(el, "background") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  auxFondo.cycle = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "red") == 0) {
				  auxFondo.red = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "green") == 0) {
				  auxFondo.green = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "blue") == 0) {
				  auxFondo.blue = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "file") == 0) {
				  strcpy(auxFondo.file, data->anim->convertirRuta(attr[i+1])); 
			  } else if (strcmp(attr[i], "position") == 0) {
				  if (strcmp(attr[i+1], "top-left") == 0) {
					  auxFondo.background_align = BG_TOP_LEFT;
				  } else if (strcmp(attr[i+1], "top-right") == 0) {
					  auxFondo.background_align = BG_TOP_RIGHT;
				  } else if (strcmp(attr[i+1], "bottom-left") == 0) {
					  auxFondo.background_align = BG_BOTTOM_LEFT;
				  } else if (strcmp(attr[i+1], "bottom-right") == 0) {
					  auxFondo.background_align = BG_BOTTOM_RIGHT;
				  } else if (strcmp(attr[i+1], "mosaic") == 0) {
					  auxFondo.background_align = BG_MOSAIC;
				  } else {
					  auxFondo.background_align = BG_CENTER;
				  }
			  } else if (strcmp(attr[i], "image-with-alpha") == 0) {
				  if (strcmp(attr[i+1], "true")==0) {
					  data->image_with_alpha = 1;
			  	  }
		  	  }
		  }
		} else if (strcmp(el, "foreground") == 0) {
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "red") == 0) {
				  data->fg.red = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "green") == 0) {
				  data->fg.green = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "blue") == 0) {
				  data->fg.blue = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "file") == 0) {
				  strcpy(data->fg.file, data->anim->convertirRuta(attr[i+1])); 
			  } else if (strcmp(attr[i], "position") == 0) {
				  if (strcmp(attr[i+1], "top-left") == 0) {
					  data->fg.background_align = BG_TOP_LEFT;
				  } else if (strcmp(attr[i+1], "top-right") == 0) {
					  data->fg.background_align = BG_TOP_RIGHT;
				  } else if (strcmp(attr[i+1], "bottom-left") == 0) {
					  data->fg.background_align = BG_BOTTOM_LEFT;
				  } else if (strcmp(attr[i+1], "bottom-right") == 0) {
					  data->fg.background_align = BG_BOTTOM_RIGHT;
				  } else if (strcmp(attr[i+1], "mosaic") == 0) {
					  data->fg.background_align = BG_MOSAIC;
				  } else {
					  data->fg.background_align = BG_CENTER;
				  }
			  } else if (strcmp(attr[i], "background-transparent") == 0) {
				if (strcmp(attr[i+1], "true") == 0) {
					data->background_transparent = 1;
				} else {
					data->background_transparent = 0;
				}
			  } else if (strcmp(attr[i], "image-with-alpha") == 0) {
				  if (strcmp(attr[i+1], "true")==0) {
					  data->image_with_alpha = 1;
			  	  }
		  	  }
		  }
		} else if ( (strcmp(el, "data") == 0) || (strcmp(el, "text-stop") == 0) ){ // guardamos los parámetros de data
		  for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "volumen") == 0) {
				  data->volumen = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "align") == 0) {
				  if (strcmp(attr[i+1], "center") == 0) {
					  data->align = 1;
				  } else if (strcmp(attr[i+1], "right") == 0) {
					  data->align = 2;
				  } else {
					  data->align = 0;
				  }
			  } else if (strcmp(attr[i], "valign") == 0) {
				  if (strcmp(attr[i+1], "center") == 0) {
					  data->valign = 1;
				  } else if (strcmp(attr[i+1], "bottom") == 0) {
					  data->valign = 2;
				  } else {
					  data->valign = 0;
				  }
			  } else if (strcmp(attr[i], "text-align") == 0) {
				  if (strcmp(attr[i+1], "center") == 0) {
					  data->text_align = 1;
				  } else if (strcmp(attr[i+1], "right") == 0) {
					  data->text_align = 2;
				  } else {
					  data->text_align = 0;
				  }
			  } else if (strcmp(attr[i], "file") == 0) {
				  strcpy(data->file, data->anim->convertirRuta(attr[i+1])); 
			  } else if (strcmp(attr[i], "text") == 0) {
				  strcpy(data->text, attr[i+1]);
			  } else if (strcmp(attr[i], "font-ttf") == 0) {
				  strcpy(data->font_ttf, data->anim->convertirRuta(attr[i+1])); 
			  } else if (strcmp(attr[i], "font-size") == 0) {
				  data->font_size = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "background-transparent") == 0) {
				if (strcmp(attr[i+1], "true") == 0) {
					data->background_transparent = 1;
				} else {
					data->background_transparent = 0;
				}
			  } else if (strcmp(attr[i], "separate-line") == 0) {
				  data->separate_line = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "x") == 0) {
				  data->x = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "y") == 0) {
				  data->y = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "fgR") == 0) {
				  data->fgR = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "fgG") == 0) {
				  data->fgG = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "fgB") == 0) {
				  data->fgB = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "bgR") == 0) {
				  data->bgR = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "bgG") == 0) {
				  data->bgG = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "bgB") == 0) {
				  data->bgB = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "cycles-between-frames") == 0) {
				  data->cycles_between_frames = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "rotation-circle") == 0) {
				  if (strcmp(attr[i+1], "true")==0) {
					  data->rotation_circle = 1;
				  }
			  } else if (strcmp(attr[i], "image-with-alpha") == 0) {
				  if (strcmp(attr[i+1], "true")==0) {
					  data->image_with_alpha = 1;
				  }
			  } else if (strcmp(attr[i], "rotate") == 0) {
				  data->rotateInicial = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "zoom") == 0) {
				  data->zoomInicial = atof(attr[i+1]);
			  } else if (strcmp(attr[i], "cycles-between-frames") == 0) {
				  data->cycles_between_frames = atoi(attr[i+1]);
			  }
		  }
		} else if (strcmp(el, "stop-animation") == 0) {
			auxControlAnimation.isPlay = 0;
			for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  auxControlAnimation.cycle = atoi(attr[i+1]);
			  } else if (strcmp(attr[i], "frame") == 0) {
				  auxControlAnimation.frame = atoi(attr[i+1]);
			  }
		  	}
		} else if (strcmp(el, "play-animation") == 0) {
			auxControlAnimation.isPlay = 1;
			for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  auxControlAnimation.cycle = atoi(attr[i+1]);
			  }
		  	}
		} else if (strcmp(el, "motion") == 0) { // comenzamos a guardar un movimiento
		  data->datMov = 1;
		} else if (strcmp(el, "location") == 0) { // comenzamos a guardar localización
		  data->datLoc = 1;
		} else if (strcmp(el, "frames") == 0) { // comenzamos a guardar frames
		  data->datFrame = 1;
		} else if (strcmp(el, "disappear") == 0) { // comenzamos a guardar un movimiento
			for (i = 0; attr[i]; i += 2) {
			  if (strcmp(attr[i], "cycle") == 0) {
				  data->cycle_disappear = atoi(attr[i+1]);
			  }
		  	}
		} else if (strcmp(el, "sounds") == 0) { // comenzamos a guardar esquemas de sonido
		  data->datSonido = 1;
		}
	}
	
  
}

static void endXML(void *userData, const char *el)
{
	UserData* data = (UserData*)userData;
	int i;
	int coord_end_x, coord_end_y;
	
	if (strcmp(el, "object") == 0) { // si es el final del objeto, grabamos y reiniciamos todo
		// crear elementos
		if (strcmp(data->type, "image") == 0) {
			data->anim->crearElemento(data->name, data->file, data->image_with_alpha);
		} else if (strcmp(data->type, "animation") == 0) {
			data->anim->crearElementoAnimacion(data->name, data->numFrames, data->cycles_between_frames, data->rotation_circle);
		} else if (strcmp(data->type, "text") == 0) {
			data->anim->crearTexto(data->name, data->text, data->font_size, data->font_ttf, data->text_align, data->background_transparent, data->separate_line, 0, 0, data->fgR, data->fgG, data->fgB, data->bgR, data->bgG, data->bgB);
		} else if (strcmp(data->type, "sound") == 0) {
			data->anim->crearSonido(data->name, data->file, data->volumen);
		}
		if (strcmp(data->type, "foreground") != 0) { // si no es un foreground
			// añadimos esquemas a los sonidos si hay
			for (i=0; i<data->son.size(); i++) {
				data->anim->addEsquema(data->name, data->son[i].cycle_in, data->son[i].cycle_out, data->son[i].time_fadein, data->son[i].time_fadeout, data->son[i].loops);
			}
			// añadimos frames de la animacion si es necesario
			for (i=0; i<data->frames.size(); i++) {
				data->anim->addImagenElementoAnimacion(data->name, data->frames[i].file, data->frames[i].image_with_alpha);
			}
			// añadimos controles de animacion si es necesario
			for (i=0; i<data->controlAnimation.size(); i++) {
				data->anim->addControlAnimation(data->name, data->controlAnimation[i]);
			}
			// añadimos fondos
			for (i=0; i<data->numFondos; i++) {
				data->anim->addFondo(data->fondos[i].cycle, data->fondos[i].red, data->fondos[i].green, data->fondos[i].blue, data->fondos[i].file, data->fondos[i].image_with_alpha, data->fondos[i].background_align);
			}
			// añadimos rotacion inicial y zoom inicial si lo hay
			if (data->rotateInicial != 0) {
				data->anim->addRotateInicial(data->name, data->rotateInicial);
			}
			if (data->zoomInicial != 1) {
				data->anim->addZoomInicial(data->name, data->zoomInicial);
			}
			// añadir movimientos
			for (i=0; i<data->mov.size(); i++) {
				if (data->mov[i].coord_end_x == 999999) {
					coord_end_x = data->mov[i].coord_begin_x;
				} else {
					coord_end_x = data->mov[i].coord_end_x;
				}
				if (data->mov[i].coord_end_y == 999999) {
					coord_end_y = data->mov[i].coord_begin_y;
				} else {
					coord_end_y = data->mov[i].coord_end_y;
				}
				
				if ( (data->mov[i].fadein <= 0) && (data->mov[i].fadeout <= 0) ) {
					data->anim->addMovimiento(data->name, data->mov[i].cycle_begin, data->mov[i].cycle_end, data->mov[i].coord_begin_x, data->mov[i].coord_begin_y, coord_end_x, coord_end_y, data->mov[i].disappear_to_end);
				} else {
					if ( (data->mov[i].fadein >0) && (data->mov[i].fadeout >0) ) {
						data->anim->addMovimiento_FadeIn_FadeOut(data->name, data->mov[i].cycle_begin, data->mov[i].cycle_end, data->mov[i].coord_begin_x, data->mov[i].coord_begin_y, coord_end_x, coord_end_y, data->mov[i].disappear_to_end, data->mov[i].fadein, data->mov[i].fadeout, data->mov[i].fadein_factor_begin, data->mov[i].fadein_factor_end, data->mov[i].fadeout_factor_begin, data->mov[i].fadeout_factor_end);
					} else if ( data->mov[i].fadein >0) {
						data->anim->addMovimiento_FadeIn(data->name, data->mov[i].cycle_begin, data->mov[i].cycle_end, data->mov[i].coord_begin_x, data->mov[i].coord_begin_y, coord_end_x, coord_end_y, data->mov[i].disappear_to_end, data->mov[i].fadein, data->mov[i].fadein_factor_begin, data->mov[i].fadein_factor_end);
					} else if (data->mov[i].fadeout >0)  {
						data->anim->addMovimiento_FadeOut(data->name, data->mov[i].cycle_begin, data->mov[i].cycle_end, data->mov[i].coord_begin_x, data->mov[i].coord_begin_y, coord_end_x, coord_end_y, data->mov[i].disappear_to_end, data->mov[i].fadeout, data->mov[i].fadeout_factor_begin, data->mov[i].fadeout_factor_end);
					}
				}
				// cortinillas
				if (data->mov[i].curtainVIn >0)  {
					data->anim->addCortinillaIn_Vertical(data->name, data->mov[i].curtainVIn, data->mov[i].curtainVInUpDown);
				} 
				if (data->mov[i].curtainHIn >0)  {
					data->anim->addCortinillaIn_Horizontal(data->name, data->mov[i].curtainHIn, data->mov[i].curtainHInLeftRight);
				}
				if (data->mov[i].curtainVOut >0)  {
					data->anim->addCortinillaOut_Vertical(data->name, data->mov[i].curtainVOut, data->mov[i].curtainVOutUpDown);
				} 
				if (data->mov[i].curtainHOut >0)  {
					data->anim->addCortinillaOut_Horizontal(data->name, data->mov[i].curtainHOut, data->mov[i].curtainHOutLeftRight);
				}
				// zoom
				if (data->mov[i].zoomin >0)  {
					data->anim->addZoomIn(data->name, data->mov[i].zoomin, data->mov[i].zoomin_factor_begin, data->mov[i].zoomin_factor_end);
				} 
				if (data->mov[i].zoomout >0)  {
					data->anim->addZoomOut(data->name, data->mov[i].zoomout, data->mov[i].zoomout_factor_begin, data->mov[i].zoomout_factor_end);
				} 
				// rotaciones
				if (data->mov[i].rotatein >0)  {
					data->anim->addRotateIn(data->name, data->mov[i].rotatein, data->mov[i].rotatein_angle_begin, data->mov[i].rotatein_angle_end, data->mov[i].factorRotatein);
					
				}
				if (data->mov[i].rotateout >0)  {
					data->anim->addRotateOut(data->name, data->mov[i].rotateout, data->mov[i].rotateout_angle_begin, data->mov[i].rotateout_angle_end, data->mov[i].factorRotateout);
					
				}
				// alineacion
				if ( (data->align != 0) || (data->valign != 0) ) {
					data->anim->addAlineacion(data->name, data->align, data->valign);
				}
				// si no es un texto, y insertamos los colores de fondo que se utilizarán
				// para los fondos de los alpha en el caso que la imagen PNG contenga alpha
				if (strcmp(data->type, "text") != 0) {
					data->anim->addColoresFondoEnAlpha(data->name, data->bgR, data->bgG, data->bgB);
				}			
				//printf("objeto: %s ciclos: %d %d begin: %d %d end: %d %d desaparecer: %d fade: %d %d\n", data->name, data->mov[i].cycle_begin, data->mov[i].cycle_end, data->mov[i].coord_begin_x, data->mov[i].coord_begin_y, data->mov[i].coord_end_x, data->mov[i].coord_end_y, data->mov[i].disappear_to_end, data->mov[i].fadein, data->mov[i].fadeout);
			}
			// añadir desaparicion del objeto en el caso que haya
			if (data->cycle_disappear != -1) {
				data->anim->addDesaparicion (data->name, data->cycle_disappear);
			}
		} else { // si es un foreground
			data->anim->addForeground(data->mov[0].cycle_begin, data->mov[0].cycle_end, data->fg.red, data->fg.green, data->fg.blue, data->fg.file, data->fg.image_with_alpha, data->fg.background_align, data->background_transparent);
			// fade
			if ( (data->mov[0].fadein >0) && (data->mov[0].fadeout >0) ) {
				data->anim->addForeground_FadeIn(data->mov[0].fadein);
				data->anim->addForeground_FadeOut(data->mov[0].fadeout);
			} else if (data->mov[0].fadein >0)  {
				data->anim->addForeground_FadeIn(data->mov[0].fadein);
			} else if (data->mov[0].fadeout >0)  {
				data->anim->addForeground_FadeOut(data->mov[0].fadeout);
			}
			// zoom
			if (data->mov[0].zoomin >0) {
				data->anim->addForeground_ZoomIn(data->mov[0].zoomin, data->mov[0].zoomin_factor_begin, data->mov[0].zoomin_factor_end);
			}
			if (data->mov[0].zoomout >0) {
				data->anim->addForeground_ZoomOut(data->mov[0].zoomout, data->mov[0].zoomout_factor_begin, data->mov[0].zoomout_factor_end);
			}
			// cortinillas
			if (data->mov[0].curtainVIn >0) {
				data->anim->addForeground_CurtainVIn(data->mov[0].curtainVIn);
			}
			if (data->mov[0].curtainHIn >0) {
				data->anim->addForeground_CurtainHIn(data->mov[0].curtainHIn);
			}
			if (data->mov[0].curtainVOut >0) {
				data->anim->addForeground_CurtainVOut(data->mov[0].curtainVOut);
			}
			if (data->mov[0].curtainHOut >0) {
				data->anim->addForeground_CurtainHOut(data->mov[0].curtainHOut);
			}
		}
		data->numObjects++;
		//iniciarUserData(data);
	} else if (strcmp(el, "motion") == 0) { // si es el final de un nuevo movimiento, actualizamos
		data->mov.push_back(auxMotion);
		iniciarAuxiliares();
		data->numMov++;
		data->datMov = 0;
	} else if (strcmp(el, "control") == 0) { // final de control. Guardamos el texto en el Surface
		data->anim->crearTexto("text-stop", data->text, data->font_size, data->font_ttf, data->text_align, data->background_transparent, data->separate_line, data->x, data->y, data->fgR, data->fgG, data->fgB, data->bgR, data->bgG, data->bgB);
		// alineacion
		data->anim->textStop->setAlign(data->align);
		data->anim->textStop->setVAlign(data->valign);
		for (i=0; i<data->stop.size(); i++) {
			data->anim->stop.push_back(data->stop[i]);
		}
	} else if (strcmp(el, "location") == 0) {
		data->numMov++;
		data->datLoc = 0;	
	} else if (strcmp(el, "frames") == 0) {
		data->datFrame = 0;
	} else if (strcmp(el, "frames") == 0) {
		data->datFrame = 0;
	} else if (strcmp(el, "sounds") == 0) {
		data->datSonido = 0;
	} else if (strcmp(el, "frame") == 0) {
		data->frames.push_back(auxFrame);
		iniciarAuxiliares();
		data->numFrames++;
	} else if (strcmp(el, "sound") == 0) {
		data->son.push_back(auxSonido);
		iniciarAuxiliares();
		data->numSon++;
	} else if (strcmp(el, "stop-animation") == 0) {
		data->controlAnimation.push_back(auxControlAnimation);
		iniciarAuxiliares();
	} else if (strcmp(el, "play-animation") == 0) {
		data->controlAnimation.push_back(auxControlAnimation);
		iniciarAuxiliares();
	} else if (strcmp(el, "background") == 0) {
		data->fondos.push_back(auxFondo);
		iniciarAuxiliares();
		data->numFondos++;
	} else if (strcmp(el, "variable") == 0) {
		data->anim->inVariable(auxVariable.name, auxVariable.value);
		data->numVariables++;
	} else if (strcmp(el, "stop") == 0) {
		data->numStop++;
	}
}


void parseXML(SDL_Animation *anim, char archXML[64]) {
	//char buffer[16384];
	string buffer;
	int done;
	
	UserData data;
	data.anim = anim;
	data.numMov = 0;
	data.numSon = 0;
	data.numObjects = 0;
	iniciarUserData(&data);
	iniciarAuxiliares();
	
	XML_Parser p = XML_ParserCreate(NULL);
	if (! p) {
	fprintf(stderr, "Podría no tener suficiente memoria para el parser\n");
	}

	XML_SetUserData(p, &data);	
	XML_SetElementHandler(p, startXML, endXML);

  	FILE *file = fopen(archXML, "r");
	if(!file)
		printf("Error abriendo archivo XML\n");
	
	//do
	//{
		//size_t len = fread(buffer, 1, sizeof(buffer), file);
		size_t len = buffer.length();
		done = len < sizeof(buffer);
		//printf("%s\n", buffer);
		if(!XML_Parse(p, buffer.data(), len, done)){
			printf("Error realizando el parse\n");
		}
			//parse_error(&data, XML_ErrorString(XML_GetErrorCode(data.parser)));
	//}
	//while(!done);
	fclose(file);
}

void parseBuffer(SDL_Animation *anim, string buffer) {
	int done;
	UserData data;
	data.anim = anim;
	data.numMov = 0;
	data.numSon = 0;
	data.numObjects = 0;
	iniciarUserData(&data);
	iniciarAuxiliares();

	XML_Parser p = XML_ParserCreate(NULL);
	if (! p) {
	fprintf(stderr, "Podría no tener suficiente memoria para el parser\n");
	}

	XML_SetUserData(p, &data);	
	XML_SetElementHandler(p, startXML, endXML);
	
	size_t len = buffer.length();
	done = len < sizeof(buffer);
	if(!XML_Parse(p, buffer.data(), len, done)){
		printf("Error realizando el parse\n");
	}

}
