/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */

#include "disparojugador.h"

 /*****************************
**
** CLASE DISPAROJUGADOR
**
******************************/

DisparoJugador::DisparoJugador() {
	Frame frDisparaJugador0, frDisparaJugador1;
	int i;
	char tmpch[127];
	 
	// sprites de la explosión de gaviotas 1
	sprintf (tmpch, "%s/balas.tim", DATA_DIR);
	frDisparaJugador0.loadFromRAR(tmpch, "bala1.timp");		
	frDisparaJugador1.loadFromRAR(tmpch, "bala2.timp");		
	 
	for (i=0; i < MAXBALAS; i++) {
		disparos[i] = new Elemento(2);
		disparos[i]->addFrame(frDisparaJugador0);
		disparos[i]->addFrame(frDisparaJugador1);
	}
	
	// cargamos balones del cádiz oeeeee
	
	Frame frBalon0, frBalon1, frBalon2;

	sprintf (tmpch, "%s/balas.tim", DATA_DIR);
	frBalon0.loadFromRAR(tmpch, "balon0.timp");
	frBalon1.loadFromRAR(tmpch, "balon1.timp");
	frBalon2.loadFromRAR(tmpch, "balon2.timp");
	
	for (i=0; i < MAX_BALON; i++) {
		balon[i] = new Balon(3);
		balon[i]->addFrame(frBalon0);
		balon[i]->addFrame(frBalon1);
		balon[i]->addFrame(frBalon2);
		balon[i]->setTipoElemento(BALON);
		balon[i]->selFrame(0);
	}
	
	numDisparo2 = NUMERODISPAROS2;
	
}


int DisparoJugador::elementoLibre () {
	int libre = -1, i;

	for (i=0; i < MAXBALAS; i++) {
		if (disparos[i]->getActivo() == 0) {
			libre = i;
		}
	}
	
	return libre;
}

void DisparoJugador::crear (Audio *sonido, Jugador *j, Juego *juego) {
	int numLibre;
	
	switch (juego->getTipoDisparo()) {
		case 0:
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+16);
				disparos[numLibre]->setY(j->getY() +10);
				disparos[numLibre]->setDX(0);
				disparos[numLibre]->setDY(-25);
				disparos[numLibre]->selFrame(0);
			}
		break;
		case 1:
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+12);
				disparos[numLibre]->setY(j->getY() +20);
				disparos[numLibre]->setDX(0);
				disparos[numLibre]->setDY(-35);
				disparos[numLibre]->selFrame(1);
			}
		break;
		case 2:
			// reducimos disparo 2
			numDisparo2--;
			if (numDisparo2<0) {
				juego->setTipoDisparo(1);
				numDisparo2 = NUMERODISPAROS2;
			}
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+16);
				disparos[numLibre]->setY(j->getY() +10);
				disparos[numLibre]->setDX(0);
				disparos[numLibre]->setDY(-25);
				disparos[numLibre]->selFrame(0);
			}
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+8);
				disparos[numLibre]->setY(j->getY() +10);
				disparos[numLibre]->setDX(-10);
				disparos[numLibre]->setDY(-20);
				disparos[numLibre]->selFrame(0);
			}
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+22);
				disparos[numLibre]->setY(j->getY() +10);
				disparos[numLibre]->setDX(10);
				disparos[numLibre]->setDY(-20);
				disparos[numLibre]->selFrame(0);
			}
		break;
	}

	if ( ( juego->getPoderActivo() ) && (juego->getPoderSel() == 3) ) {
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()+70);
				disparos[numLibre]->setY(j->getY() + 80);
				disparos[numLibre]->setDX(0);
				disparos[numLibre]->setDY(-25);
				disparos[numLibre]->selFrame(0);
			}
			numLibre = elementoLibre();
			if (numLibre != -1) {
				disparos[numLibre]->setActivo(1);
				disparos[numLibre]->setX(j->getX()-30);
				disparos[numLibre]->setY(j->getY() + 80);
				disparos[numLibre]->setDX(0);
				disparos[numLibre]->setDY(-25);
				disparos[numLibre]->selFrame(0);
			}
	}
	
	sonido->playSonido(SND_DISPARO);
	
}

void DisparoJugador::mover (SDL_Surface *screen) {
	int i;
	for (i=0; i < MAXBALAS; i++) {
		if (disparos[i]->getActivo() == 1) {
			disparos[i]->addDX();	
			disparos[i]->addDY();	
			disparos[i]->dibujar(screen);
			// comprobar si la bala ha salido de la pantalla para eliminarla
			if ( disparos[i]->getY() < - disparos[i]->getAlto() ) { 
				disparos[i]->setActivo(0);
			}
		}
	}
}


 void DisparoJugador::iniciar() {
	int i;
	for (i=0; i < MAXBALAS; i++) {
		disparos[i]->setActivo(0);
	}
	numDisparo2 = NUMERODISPAROS2;
}
