/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */
 
 /*****************************
**
** CLASE FUENTE
**
******************************/

#include "fuente.h"

 Fuente::Fuente() {
	salir = 0;
}
 
SDL_RWops* Fuente::RWFromRar(char *fileRar, char *file)
{
  char *data_ptr;
  unsigned long data_size;
  SDL_RWops *rwops;

  if(urarlib_get(&data_ptr, &data_size, file, fileRar, "password")) 
  {
	rwops = SDL_RWFromMem(data_ptr, data_size);
    //if(data_ptr != NULL) free(data_ptr);
  } else 
  { 
    printf("Error al descomprimir %s - %s!\n", fileRar, file); 
  }
  return rwops;
}


void Fuente::initFuente() {
	salir = 0;
	if (TTF_Init() < 0) {
		printf ("No se puede iniciar SDL_TTF: %s\n", SDL_GetError());
		salir = 1;
	}
	
}



void Fuente::escribirTexto (SDL_Surface *sc, int tipoFuente, int align, int valign, int size, char txt[63], int fgR, int fgG, int fgB, int bgR, int bgG, int bgB, int x, int y) {
if (strcmp(txt, "") != 0) {
	char tmpf[128];
	TTF_Font *fuente;
	SDL_Color fgcolor, bgcolor;
	SDL_Surface *ttext;
	SDL_Rect rect_dest, rect_src;

	
	switch(tipoFuente) {
		case 0:
			sprintf (tmpf, "%s/gilgon__.tttf", DATA_DIR);
		break;
		case 1:
			sprintf (tmpf, "%s/kmkdspb_.tttf", DATA_DIR);
		break;
		case 2:
			sprintf (tmpf, "%s/toontime.tttf", DATA_DIR);
		break;
	}
	
	fuente = TTF_OpenFont(tmpf, size);
		
	fgcolor.r = fgR;
	fgcolor.g = fgG;
	fgcolor.b = fgB;
	bgcolor.r = bgR;
	bgcolor.g = bgG;
	bgcolor.b = bgB;

	//ttext = TTF_RenderText_Shaded (fuente, txt, fgcolor, bgcolor);
	ttext = TTF_RenderUTF8_Shaded (fuente, txt, fgcolor, bgcolor);
	switch (align)
	{
		case  1:
			rect_dest.x = x - ttext->w/2;
			break;
		case  2:
			rect_dest.x = x - ttext->w;
			break;
		default:
			rect_dest.x = x;
			break;
	}
	
	switch (valign)
	{
		case  1:
			rect_dest.y = y - ttext->h/2;
			break;
		case  2:
			rect_dest.y = y - ttext->h;
			break;
		default:
			rect_dest.y = y;
			break;
	}
	rect_src.x = 0;
	rect_src.y = 0;
	rect_src.w = ttext->w;

	rect_src.h = ttext->h;
	SDL_SetColorKey ( ttext, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB (ttext->format, bgR,bgG,bgB) );
	
	SDL_BlitSurface ( ttext, NULL, sc, &rect_dest );
	
	TTF_CloseFont(fuente);
	SDL_FreeSurface (ttext);

}	
}


SDL_Surface* Fuente::escribirTexto (int tipoFuente, int size, char txt[63], int fgR, int fgG, int fgB, int bgR, int bgG, int bgB) {
	if (strcmp(txt, "") != 0) {
		char tmpf[128];
		TTF_Font *fuente;
		SDL_Color fgcolor, bgcolor;
		SDL_Surface *ttext;
		
		switch(tipoFuente) {
			case 0:
				sprintf (tmpf, "%s/gilgon__.tttf", DATA_DIR);
			break;
			case 1:
				sprintf (tmpf, "%s/kmkdspb_.tttf", DATA_DIR);
			break;
			case 2:
				sprintf (tmpf, "%s/toontime.tttf", DATA_DIR);
			break;
		}
		
		fuente = TTF_OpenFont(tmpf, size);
		
		fgcolor.r = fgR;
		fgcolor.g = fgG;
		fgcolor.b = fgB;
		bgcolor.r = bgR;
		bgcolor.g = bgG;
		bgcolor.b = bgB;
	
		
		//ttext = TTF_RenderText_Shaded (fuente, txt, fgcolor, bgcolor);
		ttext = TTF_RenderUTF8_Shaded (fuente, txt, fgcolor, bgcolor);
		
		SDL_SetColorKey ( ttext, SDL_SRCCOLORKEY | SDL_RLEACCEL, SDL_MapRGB (ttext->format, bgR,bgG,bgB) );
		
		TTF_CloseFont(fuente);
		
		return ttext;
	
	}	
}

void Fuente::escribirParrafo (SDL_Surface *sc, int tipoFuente, int align, int valign, int size, char txt[127], int fgR, int fgG, int fgB, int bgR, int bgG, int bgB, int x, int y, int separacionInterlineal) {
	if (strcmp(txt, "") != 0) {
		char *linea;
		
		linea = strtok( txt, "|" );
		escribirTexto(sc, tipoFuente, align, valign, size, linea, fgR, fgG, fgB, bgR, bgG, bgB, x, y);
		int cont = 1;
		while( (linea = strtok( NULL, "|" )) != NULL ) {
			escribirTexto(sc, tipoFuente, align, valign, size, linea, fgR, fgG, fgB, bgR, bgG, bgB, x, y+(separacionInterlineal*cont));
			cont++;
		}
	
	}	
}


void Fuente::unLoadFuente () {
	//TTF_CloseFont(fuente);
	//TTF_Quit();
}
