/*
 * Tirititran es un juego de naves en 2D ambientado en Cádiz
 * Copyright (C) 2004  Javier Pérez Pacheco
 *
 * Este juego tienen licencia Creative Commons y se permite
 * su modificacion y utilizacion libremente siempre y cuando se
 * cite al autor original y se comparta con la misma licencia.
 * No se permite su uso comercial.
 * 
 * Para mas informacion visite la web:
 * http://creativecommons.org/licenses/by-nc-sa/2.0/es/
 *
 * PROGRAMADOR
 * Javier Pérez Pacheco
 * Cadiz (Spain)
 * javi.pacheco@terra.es
 *
 * GRAFISMO Y 3D
 * Jesus Carrasco
 * Cadiz (Spain)
 * carrasco.carrasco@gmail.com
 *
 * MUSICA Y GRAFISMO
 * Shano Lores
 * Cadiz (Spain)
 * shano@mail.ono.es
 *
 */

#ifndef OPCIONES_H_
#define OPCIONES_H_

#include <SDL.h>
#include "defines.h"
#include "fuente.h"
#include "elementos.h"
#include "pantalla.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

class Elemento;
class Fuente;
class Frame;	
class Pantalla;
class SDL_Animation;

class Opciones {

private:
	SDL_Animation *anim;
	bool verAnimacion;
	int opcion, opcion_opciones;
	char nOpciones[NOPCIONES][25];
	Elemento *joy_conectado, *joy_noconectado, *joy_disparo;
	Elemento *joy_usar_poder, *joy_sel_poder, *joy_intercom; 
	Elemento *ele_nOpciones[NOPCIONES];
	char nOpciones_Opciones[NOPCIONES_OPCIONES][25];
	Elemento *ele_nOpciones_Opciones[NOPCIONES_OPCIONES];
	char nOpcionesMusica[4][25];
	Elemento *ele_nOpcionesMusica[4];
	char nOpcionesSonido[4][25];
	Elemento *ele_nOpcionesSonido[4];
	char nOpcionesHardware[2][25];
	Elemento *ele_nOpcionesHardware[2];
	char nOpcionesVelocidadJuego[3][25];
	Elemento *ele_nOpcionesVelocidadJuego[3];
	char nOpcionesFullScreen[2][25];
	Elemento *ele_nOpcionesFullScreen[2];
	char nOpcionesBpp[3][25];
	Elemento *ele_nOpcionesBpp[3];
	Elemento *ele_PuntNombres[5], *ele_PuntPuntos[5];
	int opcMusica, opcSonido, opcHardware, opcVelocidadJuego, opcFullScreen, opcBpp;
	Fuente *fuente;
	Elemento *fondo, *cruz, *caja, *modelo;
	Elemento *teclado, *joystick, *puntuaciones, *clave, *opciones, *creditos;
	int errorClave;
	int nivel; // 0-> raiz, 1 -> opciones, 2 -> teclado, 3->puntuaciones
	char claveFase[31];
	char tmpClaveFase[1];
	int tamClaveFase;
	int pantalla_h, pantalla_w;
	int sizeFont, sizeFontSel;
	int espaciado;
	Pantalla *p;
	int botonJoystickPulsado;
	int tiempoTextoPantalla;
	bool balancearModelo;
	float transModelo;

public:
	Opciones(Pantalla *pant);
	SDL_Surface* crearTextoConSombra(char texto[127], int tipo);
	void bajarOpcion() { if (opcion < NOPCIONES-1) { opcion++; } else { opcion=0; } }
	void subirOpcion() { if (opcion > 0) { opcion--; } else { opcion=NOPCIONES-1; } }
	void bajarOpcion_Opcion() { if (opcion_opciones < NOPCIONES_OPCIONES-1) { opcion_opciones++; } else { opcion_opciones=0; } }
	void subirOpcion_Opcion() { if (opcion_opciones > 0) { opcion_opciones--; } else { opcion_opciones=NOPCIONES_OPCIONES-1; } }
	void bajarOpcionMusica() { if (opcMusica < 3) { opcMusica++; } else { opcMusica=0; } }
	void subirOpcionMusica() { if (opcMusica > 0) { opcMusica--; } else { opcMusica=3; } }
	void bajarOpcionSonido() { if (opcSonido < 3) { opcSonido++; } else { opcSonido=0; } }
	void subirOpcionSonido() { if (opcSonido > 0) { opcSonido--; } else { opcSonido=3; } }
	void bajarOpcionHardware() { if (opcHardware < 1) { opcHardware++; } else { opcHardware=0; } }
	void subirOpcionHardware() { if (opcHardware > 0) { opcHardware--; } else { opcHardware=1; } }
	void bajarOpcionVelocidadJuego() { if (opcVelocidadJuego < 2) { opcVelocidadJuego++; } else { opcVelocidadJuego=0; } }
	void subirOpcionVelocidadJuego() { if (opcVelocidadJuego > 0) { opcVelocidadJuego--; } else { opcVelocidadJuego=2; } }
	void bajarOpcionFullScreen() { if (opcFullScreen < 1) { opcFullScreen++; } else { opcFullScreen=0; } }
	void subirOpcionFullScreen() { if (opcFullScreen > 0) { opcFullScreen--; } else { opcFullScreen=1; } }
	void bajarOpcionBpp() { if (opcBpp < 2) { opcBpp++; } else { opcBpp=0; } }
	void subirOpcionBpp() { if (opcBpp > 0) { opcBpp--; } else { opcBpp=2; } }
	int getOpcion() { return opcion; }
	int getOpcion_Opcion() { return opcion_opciones; }
	void cargarValoresConfiguracion(Pantalla *p);
	void cargarPuntuaciones();
	int escena(); // -1 sale del juego, 0 no pasa nada, + de 30 es el valor de la vel_juego,
											//	otro es el valor de escena
	void dibujar();
	void posInicialOpciones();
	void posFinalOpciones();
	void posInicialOpciones_Opciones();
	void posFinalOpciones_Opciones();
	void posInicialControles();
	void posFinalControles();
	void posInicialPuntuaciones();
	void posFinalPuntuaciones();
	void posInicialCreditos();
	void posFinalCreditos();
	void posInicialClave();
	void posFinalClave();
	bool teclaValida(SDLKey letra);
};

#endif /* OPCIONES_H_ */
